@@ -1,5 +1,5 @@
 // ClockTool.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClockTool.cc,v 1.10 2004/01/13 14:41:32 rathnor Exp $
+// $Id: ClockTool.cc 4010 2005-05-12 19:58:47Z mathias $
 
 #include "ClockTool.hh"
 
@@ -34,27 +34,32 @@
 #include "FbTk/ImageControl.hh"
 #include "FbTk/Menu.hh"
 #include "FbTk/MenuItem.hh"
+#include "FbTk/I18n.hh"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
-#include <ctime>
+#ifdef HAVE_CTIME
+  #include <ctime>
+#else
+  #include <time.h>
+#endif
 #include <string>
-#include <iostream>
-using namespace std;
+#include <typeinfo>
 
 class ClockMenuItem: public FbTk::MenuItem {
 public:
     explicit ClockMenuItem::ClockMenuItem(ClockTool &tool):
         FbTk::MenuItem(""), m_tool(tool) { 
         // determine 12/24 hour format
+        _FB_USES_NLS;
         if (m_tool.timeFormat().find("%k") != std::string::npos ||
             m_tool.timeFormat().find("%H") != std::string::npos ||
             m_tool.timeFormat().find("%T") != std::string::npos)
-            setLabel("Clock: 24h");
+            setLabel( _FBTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
         else
-            setLabel("Clock: 12h");
+            setLabel( _FBTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
     }
 
     void click(int button, int time) {
@@ -62,6 +67,9 @@ public:
         size_t pos = newformat.find("%k");
         std::string newstr;
         bool clock24hour = true;
+
+        _FB_USES_NLS;
+
         if (pos != std::string::npos)
             newstr = "%l";
         else if ((pos = newformat.find("%H")) != std::string::npos)
@@ -98,9 +106,9 @@ public:
             if (m_tool.timeFormat().find("%k") != std::string::npos ||
                 m_tool.timeFormat().find("%H") != std::string::npos ||
                 m_tool.timeFormat().find("%T") != std::string::npos)
-                setLabel("Clock: 24h");
+                setLabel( _FBTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
             else
-                setLabel("Clock: 12h");
+                setLabel( _FBTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
         
         } // else some other strange format...so we don't do anything
         FbTk::MenuItem::click(button, time);
@@ -138,6 +146,8 @@ ClockTool::ClockTool(const FbTk::FbWindow &parent,
     // attach signals
     theme.reconfigSig().attach(this);
 
+    _FB_USES_NLS;	
+
     // setup timer to update the graphics each second
     timeval delay;
     delay.tv_sec = 1;
@@ -156,7 +166,7 @@ ClockTool::ClockTool(const FbTk::FbWindow &parent,
     item->setCommand(saverc);
     menu.insert(item);
     FbTk::RefCount<FbTk::Command> editformat_cmd(new EditClockFormatCmd());
-    menu.insert("Edit Clock Format", editformat_cmd);
+    menu.insert(_FBTEXT(Toolbar, ClockEditFormat,   "Edit Clock Format",   "edit Clock Format") , editformat_cmd);
 
 
     update(0);
@@ -174,13 +184,15 @@ void ClockTool::move(int x, int y) {
 
 void ClockTool::resize(unsigned int width, unsigned int height) {
     m_button.resize(width, height);
-    renderTheme();
+    reRender();
+    m_button.clear();
 }
 
 void ClockTool::moveResize(int x, int y,
                       unsigned int width, unsigned int height) {
     m_button.moveResize(x, y, width, height);
-    renderTheme();
+    reRender();
+    m_button.clear();
 }
 
 void ClockTool::show() {
@@ -200,11 +212,17 @@ void ClockTool::update(FbTk::Subject *subj) {
     updateTime();
 
     // + 2 to make the entire text fit inside
-    std::string text;
-    for (size_t i=0; i<m_button.text().size() + 2; ++i)
-        text += '0';
+    std::string text(m_button.text().size() + 2, '0');
+
+    int new_width = m_theme.font().textWidth(text.c_str(), text.size());
+    if (new_width != m_button.width()) {
+        resize(new_width, m_button.height());
+        resizeSig().notify();
+    }
+
+    if (subj != 0 && typeid(*subj) == typeid(ToolTheme))
+        renderTheme(m_button.alpha());
 
-    resize(m_theme.font().textWidth(text.c_str(), text.size()), m_button.height());
 }
 
 unsigned int ClockTool::borderWidth() const { 
@@ -231,35 +249,44 @@ void ClockTool::updateTime() {
             return;
 
 #ifdef HAVE_STRFTIME
-        if (!strftime(time_string, 255, m_timeformat->c_str(), time_type))
+        if (!strftime(time_string, 255, m_timeformat->c_str(), time_type) || m_button.text() == time_string)
             return;
         m_button.setText(time_string);
 #else // dont have strftime so we have to set it to hour:minut
         //        sprintf(time_string, "%d:%d", );
 #endif // HAVE_STRFTIME
     }
-
-    m_button.clear();
-    m_button.updateTransparent();
 }
 
-void ClockTool::renderTheme() {
-    Pixmap old_pm = m_pixmap;
-    if (!m_theme.texture().usePixmap()) {
+// Just change things that affect the size
+void ClockTool::updateSizing() {
+    m_button.setBorderWidth(m_theme.border().width());
+    // resizes if new timeformat 
+    update(0);
+}
+
+void ClockTool::reRender() {
+    if (m_pixmap) 
+        m_screen.imageControl().removeImage(m_pixmap);
+
+    if (m_theme.texture().usePixmap()) {
+        m_pixmap = m_screen.imageControl().renderImage(width(), height(),
+                                                       m_theme.texture());
+        m_button.setBackgroundPixmap(m_pixmap);
+    } else {
         m_pixmap = 0;
         m_button.setBackgroundColor(m_theme.texture().color());
-    } else {
-        m_pixmap = m_screen.imageControl().renderImage(m_button.width(), m_button.height(), m_theme.texture());
-        m_button.setBackgroundPixmap(m_pixmap);
     }
+}
 
-    if (old_pm)
-        m_screen.imageControl().removeImage(old_pm);
 
+void ClockTool::renderTheme(unsigned char alpha) {
+    m_button.setAlpha(alpha);
     m_button.setJustify(m_theme.justify());
+
+    reRender();
+
     m_button.setBorderWidth(m_theme.border().width());
     m_button.setBorderColor(m_theme.border().color());
-    m_button.setAlpha(m_theme.alpha());
     m_button.clear();
-    m_button.updateTransparent();
 }
