@@ -1,5 +1,5 @@
 // Container.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Container.cc,v 1.13 2004/01/21 13:34:40 fluxgen Exp $
+// $Id: Container.cc 3932 2005-04-10 18:18:14Z simonb $
 
 #include "Container.hh"
 
@@ -37,7 +37,7 @@ Container::Container(const FbTk::FbWindow &parent):
 }
 
 Container::~Container() {
-
+    // ~FbWindow cleans event manager
 }
 
 void Container::resize(unsigned int width, unsigned int height) {
@@ -132,7 +132,6 @@ void Container::removeAll() {
     m_item_list.clear();
     if (!m_update_lock) {
         clear();
-        updateTransparent();
     }
 
 }
@@ -160,10 +159,7 @@ void Container::setSelected(int pos) {
         for (; pos != 0; --pos, ++it)
             continue;
         m_selected = *it;
-        if (m_selected) {
-            m_selected->clear();
-            m_selected->updateTransparent();
-        }
+        // caller does any graphics stuff if appropriate
     }
         
 }
@@ -179,7 +175,6 @@ void Container::setAlignment(Container::Alignment a) {
 void Container::exposeEvent(XExposeEvent &event) {
     if (!m_update_lock) {
         clearArea(event.x, event.y, event.width, event.height);
-        updateTransparent(event.x, event.y, event.width, event.height);
     }
 }
 
@@ -218,8 +213,7 @@ void Container::repositionItems() {
                           -borderW,
                           max_width_per_client + extra,
                           height());
-        (*it)->clear();
-        (*it)->updateTransparent();
+        // moveresize does a clear
     }
 
 }
