@@ -1,5 +1,5 @@
 // CurrentWindowCmd.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen{<a*t>}users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: CurrentWindowCmd.cc 3787 2004-12-21 16:09:36Z mathias $
+// $Id: CurrentWindowCmd.cc 3925 2005-03-23 20:36:12Z mathias $
 
 #include "CurrentWindowCmd.hh"
 
@@ -129,3 +129,8 @@ void ResizeToCmd::real_execute() {
     if (m_step_size_x > 0 && m_step_size_y > 0)
         fbwindow().resize(m_step_size_x, m_step_size_y);
 }
+
+FullscreenCmd::FullscreenCmd() { }
+void FullscreenCmd::real_execute() {
+    fbwindow().setFullscreen(!fbwindow().isFullscreen());
+}
