@@ -1,5 +1,5 @@
 // Font.cc
-// Copyright (c) 2002-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,10 +19,11 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: Font.cc 3793 2004-12-21 23:42:09Z mathias $
+//$Id: Font.cc 4013 2005-05-12 20:16:45Z mathias $
 
 
 #include "StringUtil.hh"
+#include "stringstream.hh"
 #include "Font.hh"
 #include "FontImp.hh"
 #include "I18n.hh"
@@ -69,15 +70,6 @@
 #include <typeinfo>
 #include <langinfo.h>
 
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#define FB_istringstream istringstream
-#elif HAVE_STRSTREAM
-#include <strstream>
-#define FB_istringstream istrstream
-#else
-#error "You dont have sstream or strstream headers!"
-#endif // HAVE_STRSTREAM
 
 #ifdef HAVE_CSTDLIB
   #include <cstdlib>
@@ -202,14 +194,14 @@ int extract_shadow_options(const std::string& opts,
        }
        else if ( (*token).find("offsetx=", 0) != std::string::npos ) {
            size_t s= (*token).find_first_of('=');
-           FB_istringstream o((*token).substr(s + 1, (*token).length()));
+           FbTk_istringstream o((*token).substr(s + 1, (*token).length()).c_str());
            if ( !o.eof() ) {
                o >> offx;
            }
        }
        else if ( (*token).find("offsety=", 0) != std::string::npos ) {
            size_t s= (*token).find_first_of('=');
-           FB_istringstream o((*token).substr(s + 1, (*token).length()));
+           FbTk_istringstream o((*token).substr(s + 1, (*token).length()).c_str());
            if ( !o.eof() ) {
                o >> offy;
            }
@@ -352,6 +344,7 @@ void Font::setAntialias(bool flag) {
 }
 
 bool Font::load(const std::string &name) {
+
     if (name.size() == 0)
         return false;
     // default values for font options
