@@ -1,5 +1,5 @@
 // TextButton.cc for Fluxbox Window Manager
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen[at]fluxbox.org)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,41 +19,40 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextButton.cc,v 1.5 2004/01/09 02:19:02 fluxgen Exp $
+// $Id: TextButton.cc 3961 2005-04-27 15:15:36Z simonb $
 
 #include "TextButton.hh"
 #include "Font.hh"
 #include "GContext.hh"
-#include <iostream>
-using namespace std;
 
 namespace FbTk {
 
-TextButton::TextButton(const FbTk::FbWindow &parent, 
-                       const FbTk::Font &font, 
+TextButton::TextButton(const FbTk::FbWindow &parent,
+                       const FbTk::Font &font,
                        const std::string &text):
     FbTk::Button(parent, 0, 0, 10, 10),
     m_font(&font),
     m_text(text),
     m_justify(FbTk::LEFT), m_bevel(1),
-    m_buffer(drawable(), width(), height(), depth()) {
+    m_left_padding(0),
+    m_right_padding(0) {
+    setRenderer(*this);
 
 }
 
 void TextButton::resize(unsigned int width, unsigned int height) {
-     m_buffer.resize(width, height);
+    if (this->width() == width && height == this->height())
+        return;
 
-    if (backgroundPixmap() != ParentRelative)
-        FbWindow::setBackgroundPixmap(m_buffer.drawable());
     Button::resize(width, height);
 }
 
 void TextButton::moveResize(int x, int y,
                             unsigned int width, unsigned int height) {
-    m_buffer.resize(width, height);
-    
-    if (backgroundPixmap() != ParentRelative)
-        FbWindow::setBackgroundPixmap(m_buffer.drawable());
+    if (this->width() == width && height == this->height() &&
+        x == this->x() && y == this->y())
+        return;
+
     Button::moveResize(x, y, width, height);
 }
 
@@ -62,7 +61,11 @@ void TextButton::setJustify(FbTk::Justify just) {
 }
 
 void TextButton::setText(const std::string &text) {
-    m_text = text;
+    if (m_text != text) {
+        m_text = text;
+        updateBackground(false);
+        clear();
+    }
 }
 
 void TextButton::setFont(const FbTk::Font &font) {
@@ -79,7 +82,20 @@ void TextButton::setBevel(int bevel) {
     m_bevel = bevel;
 }
 
-/// clear window and redraw text 
+void TextButton::setTextPaddingLeft(unsigned int leftpadding) {
+    m_left_padding = leftpadding;
+}
+
+void TextButton::setTextPaddingRight(unsigned int rightpadding) {
+    m_right_padding = rightpadding;
+}
+
+void TextButton::setTextPadding(unsigned int padding) {
+    setTextPaddingLeft(padding/2);
+    setTextPaddingRight(padding/2);
+}
+
+/// clear window and redraw text
 void TextButton::clear() {
     TextButton::clearArea(0, 0,
                           width(), height());
@@ -88,66 +104,43 @@ void TextButton::clear() {
 void TextButton::clearArea(int x, int y,
                            unsigned int width, unsigned int height,
                            bool exposure) {
-    if (backgroundPixmap() != ParentRelative) {
-
-        if (backgroundPixmap()) {
-            m_buffer.copyArea(backgroundPixmap(),
-                              gc(),
-                              x, y,
-                              x, y,
-                              width, height);
-
-        } else { // fill with background color
-            FbTk::GContext gc(m_buffer);        
-            gc.setForeground(backgroundColor());
-            m_buffer.fillRectangle(gc.gc(),
-                                   x, y,
-                                   width, height);
-            
-        }
-
-        drawText();
-
-        setBufferPixmap(m_buffer.drawable());
-        FbWindow::setBackgroundPixmap(m_buffer.drawable());
-        updateTransparent(x, y, width, height);
-
-        FbWindow::clearArea(x, y, width, height, exposure);
-        
-    } else { // parent relative 
-        FbWindow::setBufferPixmap(0);
-        FbWindow::setBackgroundPixmap(backgroundPixmap());     
-        Button::clearArea(x, y, width, height, exposure);
-        updateTransparent(x, y, width, height);
-        drawText();
-    }   
-
-    
-
+    Button::clearArea(x, y, width, height, exposure);
+    if (backgroundPixmap() == ParentRelative) 
+        drawText(0, 0, this);
 }
 
 unsigned int TextButton::textWidth() const {
     return font().textWidth(text().c_str(), text().size());
 }
 
-void TextButton::drawText(int x_offset, int y_offset) {
+void TextButton::renderForeground(FbWindow &win, FbDrawable &drawable) {
+    // (win should always be *this, no need to check)
+    drawText(0, 0, &drawable);
+}
+
+void TextButton::drawText(int x_offset, int y_offset, FbDrawable *drawable) {
     unsigned int textlen = text().size();
     // do text alignment
-    int align_x = FbTk::doAlignment(width() - x_offset,
+
+    int align_x = FbTk::doAlignment(width() - x_offset - m_left_padding - m_right_padding,
                                     bevel(),
                                     justify(),
                                     font(),
                                     text().c_str(), text().size(),
-                                    textlen); // return new text len
+                                    textlen); // return new text lne
 
     // center text by default
     int center_pos = height()/2 + font().ascent()/2 - 1;
 
-    font().drawText(backgroundPixmap() == ParentRelative ? window() : m_buffer.drawable(),
+    if (drawable == 0)
+        drawable = this;
+
+    font().drawText(*drawable,
                     screenNumber(),
                     gc(), // graphic context
                     text().c_str(), textlen, // string and string size
-                    align_x + x_offset, center_pos + y_offset); // position
+                    align_x + x_offset + m_left_padding, center_pos + y_offset); // position
+    
 }
 
 void TextButton::exposeEvent(XExposeEvent &event) {
