@@ -1,5 +1,5 @@
 // Theme.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,17 +19,22 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Theme.cc,v 1.25 2004/01/12 20:24:06 fluxgen Exp $
+// $Id: Theme.cc 3915 2005-03-16 22:44:48Z mathias $
 
 #include "Theme.hh"
 
 #include "XrmDatabaseHelper.hh"
 #include "App.hh"
 #include "StringUtil.hh"
-#include "ThemeItems.hh"
-#include "Directory.hh"
+#include "FileUtil.hh"
+#include "I18n.hh"
+#include "Image.hh"
 
-#include <cstdio>
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 #include <memory>
 #include <iostream>
 
@@ -82,13 +87,17 @@ bool ThemeManager::load(const std::string &filename, int screen_num) {
     std::string location = FbTk::StringUtil::expandFilename(filename);
     std::string prefix = "";
 
-    if (Directory::isDirectory(filename)) {
+    if (FileUtil::isDirectory(filename.c_str())) {
         prefix = location;
 
         location.append("/theme.cfg");
-        if (!Directory::isRegularFile(location)) {
-            cerr<<"Error loading theme file "<<location<<": not a regular file"<<endl;
-            return false;
+        if (!FileUtil::isRegularFile(location.c_str())) {
+            location = prefix;
+            location.append("/style.cfg");
+            if (!FileUtil::isRegularFile(location.c_str())) {
+                cerr<<"Error loading theme file "<<location<<": not a regular file"<<endl;
+                return false;
+            }
         }
     } else {
         // dirname
@@ -126,8 +135,10 @@ bool ThemeManager::load(const std::string &filename, int screen_num) {
     theme_it = m_themelist.begin();
     for (; theme_it != theme_it_end; ++theme_it) {
         // send reconfiguration signal to theme and listeners
-        (*theme_it)->reconfigTheme();
-        (*theme_it)->reconfigSig().notify();
+        if (screen_num < 0 || (*theme_it)->screenNum() == screen_num) {
+            (*theme_it)->reconfigTheme();
+            (*theme_it)->reconfigSig().notify();
+        }
     }
     return true;
 }
@@ -140,9 +151,11 @@ void ThemeManager::loadTheme(Theme &tm) {
         if (!loadItem(*resource)) {
             // try fallback resource in theme
             if (!tm.fallback(*resource)) {
-                if (verbose())
-                    cerr<<"Failed to read theme item: "<<resource->name()<<endl;
-                resource->setDefaultValue();                
+                if (verbose()) {
+                    _FB_USES_NLS;
+                    cerr<<_FBTKTEXT(Error, ThemeItem, "Failed to read theme item", "When reading a style, couldn't read a specific item (following)")<<": "<<resource->name()<<endl;
+                }
+                resource->setDefaultValue();
             }
         }
     }
@@ -157,11 +170,10 @@ bool ThemeManager::loadItem(ThemeItem_base &resource) {
 bool ThemeManager::loadItem(ThemeItem_base &resource, const std::string &name, const std::string &alt_name) {
     XrmValue value;
     char *value_type;
-
     if (XrmGetResource(*m_database, name.c_str(),
                        alt_name.c_str(), &value_type, &value)) {
         resource.setFromString(value.addr);
-        resource.load(); // load additional stuff by the ThemeItem
+        resource.load(&name, &alt_name); // load additional stuff by the ThemeItem
     } else
         return false;
 
@@ -171,11 +183,10 @@ bool ThemeManager::loadItem(ThemeItem_base &resource, const std::string &name, c
 std::string ThemeManager::resourceValue(const std::string &name, const std::string &altname) {
     XrmValue value;
     char *value_type;
-	
     if (*m_database != 0 && XrmGetResource(*m_database, name.c_str(),
-                                           altname.c_str(), &value_type, &value) && value.addr != 0) {
+                                           altname.c_str(), &value_type, &value) && value.addr != 0)
         return string(value.addr);
-    }
+
     return "";
 }
 
