@@ -1,6 +1,6 @@
 // XftFontImp.cc  Xft font implementation for FbTk
-// Copyright (c) 2002 Henrik Kinnunen (fluxgen@linuxmail.org)
-// 
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+//
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
 // to deal in the Software without restriction, including without limitation
@@ -19,18 +19,21 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: XftFontImp.cc,v 1.2 2002/12/01 13:42:15 rathnor Exp $
+//$Id: XftFontImp.cc 3864 2005-01-24 18:02:34Z mathias $
 
 #include "XftFontImp.hh"
 #include "App.hh"
+#include "FbDrawable.hh"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif //HAVE_CONFIG_H
+
 namespace FbTk {
 
 XftFontImp::XftFontImp(const char *name, bool utf8):m_xftfont(0),
                                                     m_utf8mode(utf8) {
+
     if (name != 0)
         load(name);
 }
@@ -42,10 +45,10 @@ XftFontImp::~XftFontImp() {
 
 bool XftFontImp::load(const std::string &name) {
     //Note: assumes screen 0 for now, changes on draw if needed
-	
+
     Display *disp = App::instance()->display();
     XftFont *newxftfont = XftFontOpenName(disp, 0, name.c_str());
-		
+
     if (newxftfont == 0) { // failed to open font, lets test with XLFD
         newxftfont = XftFontOpenXlfd(disp, 0, name.c_str());
         if (newxftfont == 0)
@@ -60,25 +63,24 @@ bool XftFontImp::load(const std::string &name) {
     return true;
 }
 
-void XftFontImp::drawText(Drawable w, int screen, GC gc, const char *text, size_t len, int x, int y) const {
+void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *text, size_t len, int x, int y) const {
     if (m_xftfont == 0)
         return;
-    Display *disp = App::instance()->display();
-    XftDraw *draw = XftDrawCreate(disp,
-                                  w,
-                                  DefaultVisual(disp, screen),
-                                  DefaultColormap(disp, screen));
+    XftDraw *draw = XftDrawCreate(w.display(),
+                                  w.drawable(),
+                                  DefaultVisual(w.display(), screen),
+                                  DefaultColormap(w.display(), screen));
 
     XGCValues gc_val;
 
     // get foreground pixel value and convert it to XRenderColor value
     // TODO: we should probably check return status
-    XGetGCValues(disp, gc, GCForeground, &gc_val);
+    XGetGCValues(w.display(), gc, GCForeground, &gc_val);
 
     // get red, green, blue values
     XColor xcol;
     xcol.pixel = gc_val.foreground;
-    XQueryColor(disp, DefaultColormap(disp, screen), &xcol);
+    XQueryColor(w.display(), DefaultColormap(w.display(), screen), &xcol);
 
     // convert xcolor to XftColor
     XRenderColor rendcol;
@@ -86,51 +88,75 @@ void XftFontImp::drawText(Drawable w, int screen, GC gc, const char *text, size_
     rendcol.green = xcol.green;
     rendcol.blue = xcol.blue;
     rendcol.alpha = 0xFFFF;
-    XftColor xftcolor;	
-    XftColorAllocValue(disp, DefaultVisual(disp, screen), DefaultColormap(disp, screen),
+    XftColor xftcolor;
+    XftColorAllocValue(w.display(), 
+                       DefaultVisual(w.display(), screen), 
+                       DefaultColormap(w.display(), screen),
                        &rendcol, &xftcolor);
 
     // draw string
 #ifdef HAVE_XFT_UTF8_STRING
     if (m_utf8mode) {
-        XftDrawStringUtf8(draw,
-                          &xftcolor,
-                          m_xftfont,
-                          x, y,
-                          (XftChar8 *)(text), len);
-    } else 
-#endif // HAVE_XFT_UTF8_STRING
-	{
-            XftDrawString8(draw,
-                           &xftcolor,
+        // check the string size,
+        // if the size is zero we use the XftDrawString8 function instead.
+        XGlyphInfo ginfo;
+        XftTextExtentsUtf8(w.display(),
                            m_xftfont,
-                           x, y,
-                           (XftChar8 *)(text), len);
-	}
+                           (XftChar8 *)text, len,
+                           &ginfo);
+        if (ginfo.xOff != 0) {
+            XftDrawStringUtf8(draw,
+                              &xftcolor,
+                              m_xftfont,
+                              x, y,
+                              (XftChar8 *)(text), len);
+            XftColorFree(w.display(), 
+                         DefaultVisual(w.display(), screen),
+                         DefaultColormap(w.display(), screen), &xftcolor);
+            XftDrawDestroy(draw);
+            return;
+        }
+    }
+#endif // HAVE_XFT_UTF8_STRING
 
-    XftColorFree(disp, DefaultVisual(disp, screen), 
-                 DefaultColormap(disp, screen), &xftcolor);
+    XftDrawString8(draw,
+                   &xftcolor,
+                   m_xftfont,
+                   x, y,
+                   (XftChar8 *)(text), len);
+
+
+    XftColorFree(w.display(), 
+                 DefaultVisual(w.display(), screen),
+                 DefaultColormap(w.display(), screen), &xftcolor);
     XftDrawDestroy(draw);
 }
 
 unsigned int XftFontImp::textWidth(const char * const text, unsigned int len) const {
     if (m_xftfont == 0)
         return 0;
+
     XGlyphInfo ginfo;
+    Display* disp = App::instance()->display();
+
 #ifdef HAVE_XFT_UTF8_STRING
     if (m_utf8mode) {
-        XftTextExtentsUtf8(App::instance()->display(),
+        XftTextExtentsUtf8(disp,
                            m_xftfont,
                            (XftChar8 *)text, len,
                            &ginfo);
-    } else 
+        if (ginfo.xOff != 0)
+            return ginfo.xOff;
+
+        // the utf8 failed, try normal extents
+    }
 #endif  //HAVE_XFT_UTF8_STRING
-	{
-            XftTextExtents8(App::instance()->display(),
-                            m_xftfont,
-                            (XftChar8 *)text, len,
-                            &ginfo);
-	}
+
+    XftTextExtents8(disp,
+                    m_xftfont,
+                    (XftChar8 *)text, len,
+                    &ginfo);
+
     return ginfo.xOff;
 }
 
@@ -138,6 +164,9 @@ unsigned int XftFontImp::height() const {
     if (m_xftfont == 0)
         return 0;
     return m_xftfont->height;
+    //m_xftfont->ascent + m_xftfont->descent;
+    // curiously, fonts seem to have a smaller height, but the "height"
+    // is specified within the actual font, so it must be right, right?
 }
 
 }; // end namespace FbTk
