@@ -1,5 +1,5 @@
 // FbWinFrameTheme.cc for Fluxbox Window Manager
-// Copyright (c) 2003-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbWinFrameTheme.cc,v 1.17 2004/01/11 16:13:09 fluxgen Exp $
+// $Id: FbWinFrameTheme.cc 3865 2005-01-24 18:34:57Z mathias $
 
 #include "FbWinFrameTheme.hh"
 #include "App.hh"
@@ -58,7 +58,6 @@ FbWinFrameTheme::FbWinFrameTheme(int screen_num):
     m_font(*this, "window.font", "Window.Font"),
     m_textjustify(*this, "window.justify", "Window.Justify"),
     m_shape_place(*this, "window.roundCorners", "Window.RoundCorners"),
-    m_alpha(*this, "window.alpha", "Window.Alpha"),
     m_title_height(*this, "window.title.height", "Window.Title.Height"),
     m_bevel_width(*this, "window.bevelWidth", "Window.BevelWidth"),
     m_handle_width(*this, "window.handleWidth", "Window.handleWidth"),
@@ -67,12 +66,13 @@ FbWinFrameTheme::FbWinFrameTheme(int screen_num):
     m_label_text_unfocus_gc(RootWindow(FbTk::App::instance()->display(), screen_num)),
     m_label_text_active_gc(RootWindow(FbTk::App::instance()->display(), screen_num)),
     m_button_pic_focus_gc(RootWindow(FbTk::App::instance()->display(), screen_num)),
-    m_button_pic_unfocus_gc(RootWindow(FbTk::App::instance()->display(), screen_num)) {
+    m_button_pic_unfocus_gc(RootWindow(FbTk::App::instance()->display(), screen_num)),
+    m_focused_alpha(255),
+    m_unfocused_alpha(255) {
 
     *m_title_height = 0;
     // set defaults
     m_font->load("fixed");
-    *m_alpha = 255;
 
     // create cursors
     Display *disp = FbTk::App::instance()->display();
@@ -106,17 +106,11 @@ bool FbWinFrameTheme::fallback(FbTk::ThemeItem_base &item) {
         return FbTk::ThemeManager::instance().loadItem(item, "window.label.unfocus.textColor", 
                                                        "Window.Label.Unfocus.TextColor");
     }
-    
 
     return false;
 }
 
 void FbWinFrameTheme::reconfigTheme() {
-    if (*m_alpha > 255)
-        *m_alpha = 255;
-    else if (*m_alpha < 0)
-        *m_alpha = 0;
-
     if (*m_bevel_width > 20)
         *m_bevel_width = 20;
     else if (*m_bevel_width < 0)
