@@ -1,5 +1,5 @@
 // Gnome.cc for fluxbox
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Gnome.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: Gnome.cc 4015 2005-05-13 11:01:01Z simonb $
 
 #include "Gnome.hh"
 
@@ -120,7 +120,7 @@ void Gnome::setupFrame(FluxboxWindow &win) {
                                  (unsigned char **) &data) && data) {
         unsigned int workspace_num = *data;
         if (win.workspaceNumber() != workspace_num) 
-            win.screen().reassociateWindow(&win, workspace_num, false);
+            win.setWorkspace(workspace_num);
         XFree (data);
     } else {
         updateWorkspace(win);
@@ -234,13 +234,13 @@ void Gnome::updateCurrentWorkspace(BScreen &screen) {
 }
 
 void Gnome::updateWorkspaceCount(BScreen &screen) {
-    int numworkspaces = screen.getCount();
+    long numworkspaces = screen.getCount();
     screen.rootWindow().changeProperty(m_gnome_wm_win_workspace_count, XA_CARDINAL, 32, PropModeReplace,
                                        (unsigned char *)&numworkspaces, 1);
 }
 
 void Gnome::updateWorkspace(FluxboxWindow &win) {
-    int val = win.workspaceNumber();
+    long val = win.workspaceNumber();
     if (win.isStuck()) {
         val = -1;
     }
@@ -259,7 +259,7 @@ void Gnome::updateWorkspace(FluxboxWindow &win) {
 
 void Gnome::updateState(FluxboxWindow &win) {
     //translate to gnome win state
-    int state=0;
+    long state=0;
     if (win.isStuck())
         state |= WIN_STATE_STICKY;
     if (win.isIconic())
@@ -279,7 +279,7 @@ void Gnome::updateState(FluxboxWindow &win) {
 void Gnome::updateLayer(FluxboxWindow &win) {
     //TODO - map from flux layers to gnome ones
     // our layers are in the opposite direction to GNOME
-    int layernum = Fluxbox::instance()->getDesktopLayer() - win.layerNum();
+    long layernum = Fluxbox::instance()->getDesktopLayer() - win.layerNum();
 
     FluxboxWindow::ClientList::iterator client_it = win.clientList().begin();
     FluxboxWindow::ClientList::iterator client_it_end = win.clientList().end();
