@@ -1,8 +1,8 @@
 // Remember.cc for Fluxbox Window Manager
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+//                     and Simon Bowden    (rathnor at users.sourceforge.net)
 // Copyright (c) 2002 Xavier Brouckaert
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
-//                and Simon Bowden    (rathnor at users.sourceforge.net)
-//
+// 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
 // to deal in the Software without restriction, including without limitation
@@ -21,7 +21,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Remember.cc 3787 2004-12-21 16:09:36Z mathias $
+// $Id: Remember.cc 4015 2005-05-13 11:01:01Z simonb $
 
 #include "Remember.hh"
 #include "ClientPattern.hh"
@@ -36,6 +36,7 @@
 #include "FbTk/StringUtil.hh"
 #include "FbTk/MenuItem.hh"
 #include "FbTk/App.hh"
+#include "FbTk/stringstream.hh"
 
 
 #include <X11/Xlib.h>
@@ -52,19 +53,6 @@
 #include <memory>
 #include <set>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#define FB_istringstream istringstream
-#elif HAVE_STRSTREAM
-#include <strstream>
-#define FB_istringstream istrstream
-#else
-#error "You dont have sstream or strstream headers!"
-#endif // HAVE_STRSTREAM
 
 using namespace std;
 
@@ -173,7 +161,7 @@ bool handleStartupItem(const string &line, int offset) {
         if (pos > 0) {
             option = str.substr(0, pos);
             if (option == "screen") {
-                FB_istringstream iss(str.c_str() + pos + 1);
+                FbTk_istringstream iss(str.c_str() + pos + 1);
                 iss >> screen;
             } else {
                 error = true;
@@ -333,7 +321,7 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                 continue; //read next line
             if (str_key == "Workspace") {
                 unsigned int w;
-                FB_istringstream iss(str_label.c_str());
+                FbTk_istringstream iss(str_label.c_str());
                 iss >> w;
                 app.rememberWorkspace(w);
             } else if (str_key == "Head") {
@@ -356,17 +344,16 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                 } else if (str_label == "MENU") {
                     l = Fluxbox::instance()->getMenuLayer();
                 } else {
-                    FB_istringstream iss(str_label.c_str());
+                    FbTk_istringstream iss(str_label.c_str());
                     iss >> l;
                 }
                 app.rememberLayer(l);
             } else if (str_key == "Dimensions") {
                 unsigned int h,w;
-                FB_istringstream iss(str_label.c_str());
+                FbTk_istringstream iss(str_label.c_str());
                 iss >> w >> h;
                 app.rememberDimensions(w,h);
             } else if (str_key == "Position") {
-                FB_istringstream iss;
                 unsigned int r= 0;
                 unsigned int x= 0;
                 unsigned int y= 0;
@@ -382,13 +369,13 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                         else if ( str_option == "CENTER" )     r= POS_CENTER;
                         else if ( str_option == "WINCENTER" )  r= POS_WINCENTER;
                         else {
-                            iss.str(str_option);
-                            iss >> r;
+                            FbTk_istringstream iss_r(str_option.c_str());
+                            iss_r >> r;
                         }
                     }
 
-                iss.str(str_label.c_str());
-                iss >> x >> y;
+                FbTk_istringstream iss_xy(str_label.c_str());
+                iss_xy >> x >> y;
                 app.rememberPosition(x, y, r);
             } else if (str_key == "Shaded") {
                 app.rememberShadedstate((str_label=="yes"));
@@ -426,7 +413,7 @@ int Remember::parseApp(ifstream &file, Application &app, string *first_line) {
                     unsigned int mask;
                     const char * str = str_label.c_str();
                     // it'll have at least one char and \0, so this is safe
-                    FB_istringstream iss(str);
+                    FbTk_istringstream iss(str);
                     // check for hex
                     if (str[0] == '0' && str[1] == 'x') {
                         iss.seekg(2);
@@ -579,7 +566,27 @@ void Remember::save() {
             apps_file << "  [Dimensions]\t{" << a.w << " " << a.h << "}" << endl;
         }
         if (a.position_remember) {
-            apps_file << "  [Position]\t{" << a.x << " " << a.y << "}" << endl;
+            apps_file << "  [Position]\t(";
+            switch(a.refc) {
+            case POS_WINCENTER:
+                apps_file << "WINCENTER";
+                break;
+            case POS_CENTER:
+                apps_file << "CENTER";
+                break;
+            case POS_LOWERLEFT:
+                apps_file << "LOWERLEFT";
+                break;
+            case POS_LOWERRIGHT:
+                apps_file << "LOWERRIGHT";
+                break;
+            case POS_UPPERRIGHT:
+                apps_file << "UPPERRIGHT";
+                break;
+            default:
+                apps_file << "UPPERLEFT";
+            }
+            apps_file << ")\t{" << a.x << " " << a.y << "}" << endl;
         }
         if (a.shadedstate_remember) {
             apps_file << "  [Shaded]\t{" << ((a.shadedstate)?"yes":"no") << "}" << endl;
@@ -821,8 +828,8 @@ void Remember::setupFrame(FluxboxWindow &win) {
     BScreen &screen = winclient.screen();
 
     if (app->workspace_remember) {
-        // TODO: fix placement to initialise properly
-        screen.reassociateWindow(&win, app->workspace, true);
+        // we use setWorkspace and not reassoc because we're still initialising
+        win.setWorkspace(app->workspace);
         if (app->jumpworkspace_remember)
             screen.changeWorkspaceID(app->workspace);
     }
