@@ -1,5 +1,5 @@
 // Screen.cc for Fluxbox Window Manager
-// Copyright (c) 2001 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Screen.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Screen.cc 3813 2005-01-05 01:56:19Z mathias $
+// $Id: Screen.cc 3990 2005-05-03 13:53:25Z simonb $
 
 
 #include "Screen.hh"
@@ -69,6 +69,8 @@
 #include "FbTk/ImageControl.hh"
 #include "FbTk/EventManager.hh"
 #include "FbTk/Transparent.hh"
+#include "FbTk/Select2nd.hh"
+#include "FbTk/Compose.hh"
 
 //use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -328,22 +330,12 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     renderPosWindow();
 
     // setup workspaces and workspace menu
-
-    if (*resource.workspaces != 0) {
-        for (int i = 0; i < *resource.workspaces; ++i) {
-            Workspace *wkspc = new Workspace(*this, m_layermanager, 
-                                             getNameOfWorkspace(m_workspaces_list.size()),
-                                             m_workspaces_list.size());
-            m_workspaces_list.push_back(wkspc);
-        }
-    } else { // create at least one workspace
-        Workspace *wkspc = new Workspace(*this, m_layermanager, 
-                                         getNameOfWorkspace(m_workspaces_list.size()),
-                                         m_workspaces_list.size());
-        m_workspaces_list.push_back(wkspc);
+    int nr_ws = *resource.workspaces;
+    addWorkspace(); // at least one
+    for (int i = 1; i < nr_ws; ++i) {
+        addWorkspace();
     }
 
-
     m_current_workspace = m_workspaces_list.front();
 
 
@@ -368,7 +360,8 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     // So we lock root theme temporary so it doesn't uses RootTheme::reconfigTheme
     // This must be fixed in the future.
     m_root_theme->lock(true);
-    FbTk::ThemeManager::instance().load(fluxbox->getStyleFilename());
+    FbTk::ThemeManager::instance().load(fluxbox->getStyleFilename(),
+                                        m_root_theme->screenNum());
     m_root_theme->lock(false);
     m_root_theme->setLineAttributes(*resource.gc_line_width,
                                     *resource.gc_line_style,
@@ -655,6 +648,7 @@ void BScreen::hideWindowMenus(const FluxboxWindow* except) {
 
 
 void BScreen::reconfigure() {
+
     m_windowtheme->setFocusedAlpha(*resource.focused_alpha);
     m_windowtheme->setUnfocusedAlpha(*resource.unfocused_alpha);
     m_menutheme->setAlpha(*resource.menu_alpha);
@@ -687,6 +681,18 @@ void BScreen::reconfigure() {
     renderGeomWindow();
     renderPosWindow();
 
+    // realize the number of workspaces from the init-file
+    const int nr_ws = *resource.workspaces;
+    if (nr_ws > m_workspaces_list.size()) {
+        while(nr_ws != m_workspaces_list.size()) {
+            addWorkspace();
+        }
+    } else if (nr_ws < m_workspaces_list.size()) {
+        while(nr_ws != m_workspaces_list.size()) {
+            removeLastWorkspace();
+        }
+    }
+
     //reconfigure menus
     m_workspacemenu->reconfigure();
     m_configmenu->reconfigure();
@@ -826,16 +832,17 @@ void BScreen::removeClient(WinClient &client) {
     for_each(getWorkspacesList().begin(), getWorkspacesList().end(),
              mem_fun(&Workspace::updateClientmenu));
 
+    using namespace FbTk;
+
     // remove any grouping this is expecting
-    Groupables::iterator it = m_expecting_groups.begin();
-    Groupables::iterator it_end = m_expecting_groups.end();
-    for (; it != it_end; ++it) {
-        if (it->second == &client) {
-            m_expecting_groups.erase(it);
-            // it should only be in there a maximum of once
-            break;
-        }
-    }
+    Groupables::iterator erase_it = find_if(m_expecting_groups.begin(),
+                                            m_expecting_groups.end(),
+                                            Compose(bind2nd(equal_to<WinClient *>(), &client),
+                                                    Select2nd<Groupables::value_type>()));
+
+    if (erase_it != m_expecting_groups.end())
+        m_expecting_groups.erase(erase_it);
+
     // the client could be on icon menu so we update it
     //!! TODO: check this with the new icon menu
     //    updateIconMenu();
@@ -850,11 +857,16 @@ void BScreen::setAntialias(bool value) {
 }
 
 int BScreen::addWorkspace() {
+
+    bool save_name = getNameOfWorkspace(m_workspaces_list.size()) != "" ? false : true;
     Workspace *wkspc = new Workspace(*this, m_layermanager, 
-                                     "",
+                                     getNameOfWorkspace(m_workspaces_list.size()),
                                      m_workspaces_list.size());
     m_workspaces_list.push_back(wkspc);
-    addWorkspaceName(wkspc->name().c_str()); // update names
+    
+    if (save_name)
+        addWorkspaceName(wkspc->name().c_str()); //update names
+
     saveWorkspaces(m_workspaces_list.size());
     
     updateNetizenWorkspaceCount();	
@@ -878,8 +890,6 @@ int BScreen::removeLastWorkspace() {
     //remove last workspace
     m_workspaces_list.pop_back();		
 
-
-
     updateNetizenWorkspaceCount();
     saveWorkspaces(m_workspaces_list.size());
     // must be deleted after we send notify!!
@@ -1170,10 +1180,8 @@ FluxboxWindow *BScreen::createWindow(Window client) {
     FbTk::App::instance()->sync(false);
 
 
-    if (isKdeDockapp(client)) {
-        if (addKdeDockapp(client)) {            
-            return 0; // dont create a FluxboxWindow for this one
-        }
+    if (isKdeDockapp(client) && addKdeDockapp(client)) {
+        return 0; // dont create a FluxboxWindow for this one
     }
 
     WinClient *winclient = new WinClient(client, *this);
@@ -1380,7 +1388,7 @@ void BScreen::nextFocus(int opts) {
                 break;
             }
 
-            FluxboxWindow *fbwin = (*it)->m_win;
+            FluxboxWindow *fbwin = (*it)->fbwindow();
             if (fbwin && !fbwin->isIconic() &&
                 (fbwin->isStuck() 
                  || fbwin->workspaceNumber() == currentWorkspaceID())) {
@@ -1475,7 +1483,7 @@ void BScreen::prevFocus(int opts) {
                 break;
             }
 
-            FluxboxWindow *fbwin = (*it)->m_win;
+            FluxboxWindow *fbwin = (*it)->fbwindow();
             if (fbwin && !fbwin->isIconic() &&
                 (fbwin->isStuck() 
                  || fbwin->workspaceNumber() == currentWorkspaceID())) {
@@ -1706,14 +1714,12 @@ void BScreen::addConfigMenu(const char *label, FbTk::Menu &menu) {
 }
 
 void BScreen::removeConfigMenu(FbTk::Menu &menu) {
-    Configmenus::iterator it = m_configmenu_list.begin();
-    Configmenus::iterator it_end = m_configmenu_list.end();
-    for (; it != it_end; ++it) {
-        if (it->second == &menu) {
-            m_configmenu_list.erase(it);
-            break;
-        }
-    }
+    Configmenus::iterator erase_it = find_if(m_configmenu_list.begin(),
+                                             m_configmenu_list.end(),
+                                             FbTk::Compose(bind2nd(equal_to<FbTk::Menu *>(), &menu),
+                                                           FbTk::Select2nd<Configmenus::value_type>()));
+    if (erase_it != m_configmenu_list.end())
+        m_configmenu_list.erase(erase_it);
     setupConfigmenu(*m_configmenu.get());
 
 }    
@@ -1823,17 +1829,17 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
         }
 
         FbTk::MenuItem *focused_alpha_item = new IntResMenuItem(_FBTEXT(Configmenu, FocusedAlpha, "Focused Window Alpha", "Transparency level of the focused window"),
-                    resource.focused_alpha, 0, 255);
+                    resource.focused_alpha, 0, 255, *alpha_menu);
         focused_alpha_item->setCommand(saverc_cmd);
         alpha_menu->insert(focused_alpha_item);
 
         FbTk::MenuItem *unfocused_alpha_item = new IntResMenuItem(_FBTEXT(Configmenu, UnfocusedAlpha, "Unfocused Window Alpha", "Transparency level of unfocused windows"),
-                    resource.unfocused_alpha, 0, 255);
+                    resource.unfocused_alpha, 0, 255, *alpha_menu);
         unfocused_alpha_item->setCommand(saverc_cmd);
         alpha_menu->insert(unfocused_alpha_item);
 
         FbTk::MenuItem *menu_alpha_item = new IntResMenuItem(_FBTEXT(Configmenu, MenuAlpha, "Menu Alpha", "Transparency level of menu"),
-                    resource.menu_alpha, 0, 255);
+                    resource.menu_alpha, 0, 255, *alpha_menu);
         menu_alpha_item->setCommand(saverc_cmd);
         alpha_menu->insert(menu_alpha_item);
 
@@ -2213,7 +2219,7 @@ FluxboxWindow *BScreen::findGroupRight(WinClient &winclient) {
         other->getGroupLeftWindow() != None)
         return 0;
 
-    return other->m_win;
+    return other->fbwindow();
 }
 void BScreen::initXinerama() {
 #ifdef XINERAMA
@@ -2364,21 +2370,6 @@ pair<int,int> BScreen::clampToHead(int head, int x, int y, int w, int h) const {
 // this can be gone and a consistent interface for the two used
 // on the actual objects
 
-/* FIXME: dead code?
-#ifdef SLIT
-template <>
-int BScreen::getOnHead<Slit>(Slit &slit) {
-    return 0;
-}
-
-template <>
-void BScreen::setOnHead<Slit>(Slit &slit, int head) {
-    //    slit.saveOnHead(head);
-    slit.reconfigure();
-}
-#endif // SLIT
-*/
-
 template<>
 void BScreen::setOnHead<FluxboxWindow>(FluxboxWindow& win, int head) {
     if (head > 0 && head <= numHeads()) {
