@@ -1,5 +1,5 @@
 // Shape.cc
-// Copyright (c) 2003-2004 Henrik Kinnunen (fluxgen(at)users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Shape.cc,v 1.11 2004/01/21 20:00:33 fluxgen Exp $
+// $Id: Shape.cc 3865 2005-01-24 18:34:57Z mathias $
 
 #include "Shape.hh"
 
@@ -28,9 +28,17 @@
 #include "FbTk/GContext.hh"
 #include "FbTk/FbPixmap.hh"
 
-#include <cstring>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
 #include <X11/Xutil.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif // SHAPE
