@@ -1,5 +1,5 @@
 // Slit.cc for fluxbox
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Slit.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Slit.cc 3770 2004-12-13 14:03:17Z mathias $
+// $Id: Slit.cc 3990 2005-05-03 13:53:25Z simonb $
 
 #include "Slit.hh"
 
@@ -295,6 +295,8 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     scr.reconfigureSig().attach(this); // if alpha changed (we disablethis signal when we get theme change sig)
 
     frame.pixmap = None;
+    // move the frame out of sight for a moment
+    frame.window.move(-frame.window.width(), -frame.window.height());
     // setup timer
     m_timer.setTimeout(200); // default timeout
     m_timer.fireOnce(true);
@@ -357,52 +359,53 @@ void Slit::updateStrut() {
         return;
     }
 
-
+    unsigned int bw = m_slit_theme->borderWidth();
     int left = 0, right = 0, top = 0, bottom = 0;
     switch (placement()) {
     case TOPLEFT:
         if (direction() == HORIZONTAL)
-            top = height();
+            top = height() + 2 * bw;
         else
-            left = width();
+            left = width() + 2 * bw;
         break;
     case TOPCENTER:
         if (direction() == HORIZONTAL)
-            top = height();
+            top = height() + 2 * bw;
         break;
     case TOPRIGHT:
         if (direction() == HORIZONTAL)
-            top = height();
+            top = height() + 2 * bw;
         else
-            right = width();
+            right = width() + 2 * bw;
         break;
     case BOTTOMLEFT:
         if (direction() == HORIZONTAL)
-            bottom = height();
+            bottom = height() + 2 * bw;
         else
-            left = width();
+            left = width() + 2 * bw;
         break;
     case BOTTOMCENTER:
         // would be strange to have it request size on vertical direction
         // each time we add a client
         if (direction() == HORIZONTAL)
-            bottom = height();
+            bottom = height() + 2 * bw;
         break;
     case BOTTOMRIGHT:
         if (direction() == HORIZONTAL)
-            bottom = height();
+            bottom = height() + 2 * bw;
         else
-            right = width();
+            right = width() + 2 * bw;
         break;
     case LEFTCENTER:
         if (direction() == VERTICAL)
-            left = width();        
+            left = width() + 2 * bw;
         break;
     case RIGHTCENTER:
         if (direction() == VERTICAL)
-            right = width();
+            right = width() + 2 * bw;
         break;
     }
+
     m_strut = screen().requestStrut(getOnHead(), left, right, top, bottom);
     screen().updateAvailableWorkspaceArea();
 }
@@ -1082,7 +1085,6 @@ void Slit::exposeEvent(XExposeEvent &ev) {
     // we don't need to clear the entire window 
     // just the are that gets exposed
     frame.window.clearArea(ev.x, ev.y, ev.width, ev.height);
-    frame.window.updateTransparent(ev.x, ev.y, ev.width, ev.height);
 }
 
 void Slit::update(FbTk::Subject *subj) {
@@ -1091,7 +1093,6 @@ void Slit::update(FbTk::Subject *subj) {
 
 void Slit::clearWindow() {
     frame.window.clear();
-    frame.window.updateTransparent();
 }
 
 void Slit::toggleHidden() {
@@ -1256,7 +1257,7 @@ void Slit::setupMenu() {
     FbTk::MenuItem *alpha_menuitem = 
         new IntResMenuItem(_FBTEXT(Common, Alpha, "Alpha", "Transparency level"),
                            m_rc_alpha,
-                           0, 255);
+                           0, 255, m_slitmenu);
     // setup command for alpha value
     MacroCommand *alpha_macrocmd = new MacroCommand(); 
     RefCount<Command> alpha_cmd(new SimpleCommand<Slit>(*this, &Slit::updateAlpha));
