@@ -1,5 +1,5 @@
 // SystemTray.cc
-// Copyright (c) 2003-2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,11 +19,12 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: SystemTray.cc,v 1.9 2004/04/19 22:49:25 fluxgen Exp $
+// $Id: SystemTray.cc 3955 2005-04-26 04:18:10Z simonb $
 
 #include "SystemTray.hh"
 
 #include "FbTk/EventManager.hh"
+#include "FbTk/ImageControl.hh"
 
 #include "AtomHandler.hh"
 #include "fluxbox.hh"
@@ -49,20 +50,31 @@ public:
     SystemTrayHandler(SystemTray &tray):m_tray(tray) {
     }
     // client message is the only thing we care about
-    bool checkClientMessage(const XClientMessageEvent &ce, 
+    bool checkClientMessage(const XClientMessageEvent &ce,
                             BScreen * screen, WinClient * const winclient) {
+        // must be on the same screen
+        if ((screen && screen->screenNumber() != m_tray.window().screenNumber()) ||
+            (winclient && winclient->screenNumber() != m_tray.window().screenNumber()) )
+            return false;
         return m_tray.clientMessage(ce);
     }
 
     void initForScreen(BScreen &screen) { };
     void setupFrame(FluxboxWindow &win) { };
-    void setupClient(WinClient &winclient) { 
+    void setupClient(WinClient &winclient) {
+        // must be on the same screen
+        if (winclient.screenNumber() != m_tray.window().screenNumber())
+            return;
+
         // we dont want a managed window
         if (winclient.fbwindow() != 0)
             return;
         // if not kde dockapp...
         if (!winclient.screen().isKdeDockapp(winclient.window()))
             return;
+        // if not our screen...
+        if (winclient.screenNumber() != m_tray.window().screenNumber())
+            return;
         winclient.setEventMask(StructureNotifyMask |
                                SubstructureNotifyMask | EnterWindowMask);
         m_tray.addClient(winclient.window());
@@ -89,38 +101,50 @@ private:
     SystemTray &m_tray;
 };
 
-SystemTray::SystemTray(const FbTk::FbWindow &parent):
+SystemTray::SystemTray(const FbTk::FbWindow& parent, ButtonTheme& theme, BScreen& screen):
     ToolbarItem(ToolbarItem::FIXED),
-    m_window(parent, 0, 0, 1, 1, ExposureMask | ButtonPressMask | ButtonReleaseMask | 
-             SubstructureNotifyMask | SubstructureRedirectMask) {
+    m_window(parent, 0, 0, 1, 1, ExposureMask | ButtonPressMask | ButtonReleaseMask |
+             SubstructureNotifyMask | SubstructureRedirectMask),
+    m_theme(theme),
+    m_screen(screen),
+    m_pixmap(0) {
 
     FbTk::EventManager::instance()->add(*this, m_window);
+    m_theme.reconfigSig().attach(this);
+
+    Fluxbox* fluxbox = Fluxbox::instance();
+    Display *disp = fluxbox->display();
 
     // setup atom name to _NET_SYSTEM_TRAY_S<screen number>
-    char intbuff[16];    
+    char intbuff[16];
     sprintf(intbuff, "%d", m_window.screenNumber());
     std::string atom_name("_NET_SYSTEM_TRAY_S");
     atom_name += intbuff; // append number
 
-    Display *disp = FbTk::App::instance()->display();
-
     // get selection owner and see if it's free
     Atom tray_atom = XInternAtom(disp, atom_name.c_str(), False);
     Window owner = XGetSelectionOwner(disp, tray_atom);
-    if (owner != 0)
+    if (owner != 0) {
+#ifdef DEBUG
+        cerr<<__FILE__<<"(SystemTray(const FbTk::FbWindow)): can't set owner!"<<endl;
+#endif // DEBUG
         return;  // the're can't be more than one owner
+    }
 
     // ok, it was free. Lets set owner
 #ifdef DEBUG
-    cerr<<__FILE__<<"("<<__FUNCTION__<<"): SETTING OWNER!"<<endl;
+    cerr<<__FILE__<<"(SystemTray(const FbTk::FbWindow)): SETTING OWNER!"<<endl;
 #endif // DEBUG
     // set owner
     XSetSelectionOwner(disp, tray_atom, m_window.window(), CurrentTime);
+
     m_handler.reset(new SystemTrayHandler(*this));
-    Fluxbox::instance()->addAtomHandler(m_handler.get(), atom_name);
-    Window root_window = RootWindow(disp, m_window.screenNumber());
+
+    fluxbox->addAtomHandler(m_handler.get(), atom_name);
+
 
     // send selection owner msg
+    Window root_window = m_screen.rootWindow().window();
     XEvent ce;
     ce.xclient.type = ClientMessage;
     ce.xclient.message_type = XInternAtom(disp, "MANAGER", False);
@@ -135,12 +159,18 @@ SystemTray::SystemTray(const FbTk::FbWindow &parent):
 
     XSendEvent(disp, root_window, false, StructureNotifyMask, &ce);
 
+    update(0);
 }
 
 SystemTray::~SystemTray() {
     // remove us, else fluxbox might delete the memory too
     Fluxbox::instance()->removeAtomHandler(m_handler.get());
     removeAllClients();
+
+    if (m_pixmap)
+        m_screen.imageControl().removeImage(m_pixmap);
+
+    // ~FbWindow cleans EventManager
 }
 
 void SystemTray::move(int x, int y) {
@@ -150,15 +180,26 @@ void SystemTray::move(int x, int y) {
 void SystemTray::resize(unsigned int width, unsigned int height) {
     if (width != m_window.width() ||
         height != m_window.height()) {
-        m_window.resize(SystemTray::width(), height);
-        rearrangeClients();
+        m_window.resize(width, height);
+        if (!m_clients.empty()) {
+            rearrangeClients();
+            resizeSig().notify();
+        }
     }
 }
 
 void SystemTray::moveResize(int x, int y,
                             unsigned int width, unsigned int height) {
-    move(x, y);
-    resize(width, height);
+    if (width != m_window.width() ||
+        height != m_window.height()) {
+        m_window.moveResize(x, y, width, height);
+        if (!m_clients.empty()) {
+            rearrangeClients();
+            resizeSig().notify();
+        }
+    } else {
+        move(x, y);
+    }
 }
 
 void SystemTray::hide() {
@@ -166,11 +207,13 @@ void SystemTray::hide() {
 }
 
 void SystemTray::show() {
+
+    update(0);
     m_window.show();
 }
 
 unsigned int SystemTray::width() const {
-    return m_clients.size()*height();
+    return m_clients.size()* (height() - 2 * m_theme.border().width());
 }
 
 unsigned int SystemTray::height() const {
@@ -186,11 +229,15 @@ bool SystemTray::clientMessage(const XClientMessageEvent &event) {
     //    static const int SYSTEM_TRAY_BEGIN_MESSAGE =  1;
     //    static const int SYSTEM_TRAY_CANCEL_MESSAGE = 2;
 
-    if (event.message_type == 
+    if (event.message_type ==
         XInternAtom(FbTk::App::instance()->display(), "_NET_SYSTEM_TRAY_OPCODE", False)) {
 
         int type = event.data.l[1];
         if (type == SYSTEM_TRAY_REQUEST_DOCK) {
+#ifndef DEBUG
+            cerr<<"SystemTray::clientMessage(const XClientMessageEvent): SYSTEM_TRAY_REQUEST_DOCK"<<endl;
+            cerr<<"window = event.data.l[2] = "<<event.data.l[2]<<endl;
+#endif // DEBUG
             addClient(event.data.l[2]);
         }
         /*
@@ -226,13 +273,24 @@ void SystemTray::addClient(Window win) {
     if (it != m_clients.end())
         return;
 
+    // make sure we have the same screen number
+    XWindowAttributes attr;
+    attr.screen = 0;
+    if (XGetWindowAttributes(FbTk::App::instance()->display(),
+                             win, &attr) != 0 &&
+        attr.screen != 0 &&
+        XScreenNumberOfScreen(attr.screen) != window().screenNumber()) {
+        return;
+    }
+
+    FbTk::FbWindow *traywin = new TrayWindow(win);
+
 #ifdef DEBUG
-    cerr<<__FILE__<<"("<<__FUNCTION__<<"): 0x"<<hex<<win<<dec<<endl;
+    cerr<<"SystemTray::addClient(Window): 0x"<<hex<<win<<dec<<endl;
 #endif // DEBUG
     if (m_clients.empty())
         show();
 
-    FbTk::FbWindow *traywin = new TrayWindow(win);
     m_clients.push_back(traywin);
     FbTk::EventManager::instance()->add(*this, win);
     FbTk::EventManager::instance()->addParent(*this, window());
@@ -240,8 +298,10 @@ void SystemTray::addClient(Window win) {
     traywin->reparent(m_window, 0, 0);
     traywin->show();
 
-    resize(width(), m_clients.size()*height());
-    
+
+#ifdef DEBUG
+    cerr<<"SystemTray: number of clients = "<<m_clients.size()<<endl;
+#endif // DEBUG
     rearrangeClients();
 }
 
@@ -251,7 +311,7 @@ void SystemTray::removeClient(Window win) {
         return;
 
 #ifdef DEBUG
-    cerr<<__FILE__<<"("<<__FUNCTION__<<"): 0x"<<hex<<win<<dec<<endl;
+    cerr<<__FILE__<<"(SystemTray::removeClient(Window)): 0x"<<hex<<win<<dec<<endl;
 #endif // DEBUG
     FbTk::FbWindow *traywin = *tray_it;
     m_clients.erase(tray_it);
@@ -259,9 +319,8 @@ void SystemTray::removeClient(Window win) {
     resize(width(), height());
     rearrangeClients();
     if (m_clients.empty()) {
-        // so we send configurenotify signal to parent
-        m_window.resize(1, 1);
-        hide();
+        // so we send notify signal to parent
+        resizeSig().notify();
     }
 }
 
@@ -272,7 +331,7 @@ void SystemTray::exposeEvent(XExposeEvent &event) {
 void SystemTray::handleEvent(XEvent &event) {
     if (event.type == DestroyNotify) {
         removeClient(event.xdestroywindow.window);
-    } else if (event.type == UnmapNotify && event.xany.send_event) { 
+    } else if (event.type == UnmapNotify && event.xany.send_event) {
         // we ignore server-generated events, which can occur
         // on restart. The ICCCM says that a client must send
         // a synthetic event for the withdrawn state
@@ -280,33 +339,84 @@ void SystemTray::handleEvent(XEvent &event) {
     } else if (event.type == ConfigureNotify) {
         // we got configurenotify from an client
         // check and see if we need to update it's size
+        // and we must reposition and resize them to fit
+        // our toolbar
         ClientList::iterator it = findClient(event.xconfigure.window);
         if (it != m_clients.end()) {
             if (static_cast<unsigned int>(event.xconfigure.width) != (*it)->width() ||
-                static_cast<unsigned int>(event.xconfigure.height) != (*it)->height())
-                (*it)->resize((*it)->width(), (*it)->height());
+                static_cast<unsigned int>(event.xconfigure.height) != (*it)->height()) {
+                // the position might differ so we update from our local
+                // copy of position
+                (*it)->moveResize((*it)->x(), (*it)->y(),
+                                  (*it)->width(), (*it)->height());
+                // this was why gaim wasn't centring the icon
+                (*it)->sendConfigureNotify(0, 0, (*it)->width(), (*it)->height());
+            }
+            // so toolbar know that we changed size
+            resizeSig().notify();
         }
+
     }
 }
 
 void SystemTray::rearrangeClients() {
-    // move and resize clients    
+    const unsigned int h = height();
+    const unsigned int bw = m_theme.border().width();
+    int final_size = m_clients.size()*h + bw;
+    resize(final_size, h);
+    update(0);
+
+    // move and resize clients
     ClientList::iterator client_it = m_clients.begin();
     ClientList::iterator client_it_end = m_clients.end();
-    int next_x = 0;
-    for (; client_it != client_it_end; ++client_it, next_x += height()) {
-        (*client_it)->moveResize(next_x, 0, height(), height());
-
-#ifdef DEBUG
-        cerr<<__FILE__<<"("<<__FUNCTION__<<"): "<<(*client_it)->width()<<", "<<(*client_it)->height()<<endl;
-#endif // DEBUG
+    int next_x = bw;
+    for (; client_it != client_it_end;
+         ++client_it, next_x += h+bw) {
+        (*client_it)->moveResize(next_x, bw, h, h);
+        (*client_it)->sendConfigureNotify(next_x, bw, h, h);
     }
+
+    client_it = m_clients.begin();
 }
 
 void SystemTray::removeAllClients() {
+    BScreen *screen = Fluxbox::instance()->findScreen(window().screenNumber());
     while (!m_clients.empty()) {
+        if (screen)
+            m_clients.back()->reparent(screen->rootWindow(), 0, 0);
         m_clients.back()->hide();
         delete m_clients.back();
         m_clients.pop_back();
     }
 }
+
+void SystemTray::update(FbTk::Subject* subject) {
+
+    if (!m_theme.texture().usePixmap()) {
+        m_window.setBackgroundColor(m_theme.texture().color());
+    }
+    else {
+        if(m_pixmap)
+            m_screen.imageControl().removeImage(m_pixmap);
+        m_pixmap = m_screen.imageControl().renderImage(width(), height(),
+                                                       m_theme.texture());
+        m_window.setBackgroundPixmap(m_pixmap);
+    }
+
+    // "themereconfigure"
+    if (subject) {
+        ClientList::iterator client_it = m_clients.begin();
+        ClientList::iterator client_it_end = m_clients.end();
+        int next_x = 0;
+        const unsigned int h = height();
+        const unsigned int b = m_theme.border().width();
+        for (; client_it != client_it_end;
+             ++client_it, next_x += h - 2 * b) {
+
+            // maybe not the best solution (yet), force a refresh of the
+            // background of the client
+            (*client_it)->hide();
+            (*client_it)->show();
+        }
+    }
+}
