@@ -1,5 +1,5 @@
 // TextTheme.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextTheme.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: TextTheme.cc 3961 2005-04-27 15:15:36Z simonb $
 
 #include "TextTheme.hh"
 
@@ -34,10 +34,6 @@ TextTheme::TextTheme(FbTk::Theme &theme,
     m_text_color(theme, name + ".textColor", altname + ".TextColor"),
     m_justify(theme, name + ".justify", altname + ".Justify"),
     m_text_gc(RootWindow(FbTk::App::instance()->display(), theme.screenNum())) {
-    *m_justify = FbTk::LEFT;
-    // set default values
-    m_font->load("fixed");
-    m_text_color->setFromString("white", theme.screenNum());
 
     update();
 }
