@@ -1,5 +1,5 @@
 // ToolFactory.cc for Fluxbox
-// Copyright (c) 2003-2005 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ToolFactory.cc 3838 2005-01-14 09:58:16Z mathias $
+// $Id: ToolFactory.cc 3961 2005-04-27 15:15:36Z simonb $
 
 #include "ToolFactory.hh"
 
@@ -85,7 +85,7 @@ ToolFactory::ToolFactory(BScreen &screen):m_screen(screen),
 }
 
 ToolbarItem *ToolFactory::create(const std::string &name, const FbTk::FbWindow &parent, Toolbar &tbar) {
-    ToolbarItem * item;
+    ToolbarItem * item = 0;
 
     unsigned int button_size = 24;
     if (tbar.theme().buttonSize() > 0)
@@ -153,9 +153,13 @@ ToolbarItem *ToolFactory::create(const std::string &name, const FbTk::FbWindow &
 
 void ToolFactory::updateThemes() {
     m_clock_theme.setAntialias(screen().antialias());
+    m_clock_theme.reconfigTheme();
     m_iconbar_theme.setAntialias(screen().antialias());
+    m_iconbar_theme.reconfigTheme();
     m_button_theme->setAntialias(screen().antialias());
+    m_button_theme->reconfigTheme();
     m_workspace_theme->setAntialias(screen().antialias());
+    m_workspace_theme->reconfigTheme();
 }
 
 
