@@ -1,5 +1,5 @@
 // Toolbar.cc for Fluxbox
-// Copyright (c) 2002 - 2004 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Toolbar.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Toolbar.cc,v 1.138 2004/01/21 13:36:09 fluxgen Exp $
+// $Id: Toolbar.cc 3990 2005-05-03 13:53:25Z simonb $
 
 #include "Toolbar.hh"
 
@@ -32,7 +32,7 @@
 // themes
 #include "ToolbarTheme.hh"
 
-#include "I18n.hh"
+#include "FbTk/I18n.hh"
 #include "fluxbox.hh"
 #include "Screen.hh"
 #include "IntResMenuItem.hh"
@@ -46,6 +46,7 @@
 #include "FbTk/EventManager.hh"
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/StringUtil.hh"
+#include "FbTk/Transparent.hh"
 
 
 // use GNU extensions
@@ -62,11 +63,13 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#include <cstring>
-#include <cstdio>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 #include <iterator>
-
-#include <iostream>
+#include <typeinfo>
 
 using namespace std;
 
@@ -101,6 +104,7 @@ setFromString(const char *strval) {
         setDefaultValue();
 }
 
+template<>
 string FbTk::Resource<Toolbar::Placement>::
 getString() {
     switch (m_value) {
@@ -112,7 +116,7 @@ getString() {
         break;
     case Toolbar::TOPCENTER:
         return string("TopCenter");
-        break;			
+        break;
     case Toolbar::BOTTOMCENTER:
         return string("BottomCenter");
         break;
@@ -152,7 +156,7 @@ public:
     SetToolbarPlacementCmd(Toolbar &tbar, Toolbar::Placement place):m_tbar(tbar), m_place(place) { }
     void execute() {
         m_tbar.setPlacement(m_place);
-        m_tbar.reconfigure();        
+        m_tbar.reconfigure();
         Fluxbox::instance()->save_rc();
     }
 private:
@@ -168,10 +172,10 @@ Toolbar::Frame::Frame(FbTk::EventHandler &evh, int screen_num):
            0, 0, // pos
            10, 10, // size
            // event mask
-           ButtonPressMask | ButtonReleaseMask | 
+           ButtonPressMask | ButtonReleaseMask | ExposureMask |
            EnterWindowMask | LeaveWindowMask | SubstructureNotifyMask,
 
-           true) // override redirect 
+           true) // override redirect
 {
 
     FbTk::EventManager &evm = *FbTk::EventManager::instance();
@@ -191,9 +195,10 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::XLayer &layer, size_t width):
     frame(*this, scrn.screenNumber()),
     m_window_pm(0),
     m_screen(scrn),
-    m_layermenu(scrn.menuTheme(), 
+    m_layeritem(frame.window, layer),
+    m_layermenu(scrn.menuTheme(),
                 scrn.imageControl(),
-                *scrn.layerManager().getLayer(Fluxbox::instance()->getMenuLayer()), 
+                *scrn.layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
                 this,
                 true),
     m_placementmenu(scrn.menuTheme(),
@@ -203,55 +208,52 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::XLayer &layer, size_t width):
                   scrn.imageControl(),
                   *scrn.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
     m_theme(scrn.screenNumber()),
-    m_layeritem(frame.window, layer),
     m_tool_factory(scrn),
     m_strut(0),
     // lock rcmanager here
-    m_rc_auto_hide(scrn.resourceManager().lock(), false, 
+    m_rc_auto_hide(scrn.resourceManager().lock(), false,
                    scrn.name() + ".toolbar.autoHide", scrn.altName() + ".Toolbar.AutoHide"),
     m_rc_maximize_over(scrn.resourceManager(), false,
                        scrn.name() + ".toolbar.maxOver", scrn.altName() + ".Toolbar.MaxOver"),
     m_rc_visible(scrn.resourceManager(), true, scrn.name() + ".toolbar.visible", scrn.altName() + ".Toolbar.Visible"),
-    m_rc_width_percent(scrn.resourceManager(), 65, 
-                       scrn.name() + ".toolbar.widthPercent", scrn.altName() + ".Toolbar.WidthPercent"),  
-    m_rc_layernum(scrn.resourceManager(), Fluxbox::Layer(Fluxbox::instance()->getDesktopLayer()), 
+    m_rc_width_percent(scrn.resourceManager(), 65,
+                       scrn.name() + ".toolbar.widthPercent", scrn.altName() + ".Toolbar.WidthPercent"),
+    m_rc_alpha(scrn.resourceManager(), 255,
+                       scrn.name() + ".toolbar.alpha", scrn.altName() + ".Toolbar.Alpha"),
+    m_rc_layernum(scrn.resourceManager(), Fluxbox::Layer(Fluxbox::instance()->getDesktopLayer()),
                   scrn.name() + ".toolbar.layer", scrn.altName() + ".Toolbar.Layer"),
     m_rc_on_head(scrn.resourceManager(), 0,
                  scrn.name() + ".toolbar.onhead", scrn.altName() + ".Toolbar.onHead"),
-    m_rc_placement(scrn.resourceManager(), Toolbar::BOTTOMCENTER, 
+    m_rc_placement(scrn.resourceManager(), Toolbar::BOTTOMCENTER,
                    scrn.name() + ".toolbar.placement", scrn.altName() + ".Toolbar.Placement"),
     m_rc_height(scrn.resourceManager(), 0, scrn.name() + ".toolbar.height", scrn.altName() + ".Toolbar.Height"),
-    m_rc_tools(scrn.resourceManager(), "workspacename, prevworkspace, nextworkspace, iconbar, systemtray, prevwindow, nextwindow, clock", 
+    m_rc_tools(scrn.resourceManager(), "workspacename, prevworkspace, nextworkspace, iconbar, systemtray, prevwindow, nextwindow, clock",
                scrn.name() + ".toolbar.tools", scrn.altName() + ".Toolbar.Tools"),
     m_shape(new Shape(frame.window, 0)),
     m_resize_lock(false) {
-
+    _FB_USES_NLS;
     // we need to get notified when the theme is reloaded
     m_theme.reconfigSig().attach(this);
     // listen to screen size changes
     screen().resizeSig().attach(this);
-    screen().reconfigureSig().attach(this); // if alpha value changes
+    screen().reconfigureSig().attach(this); // get this on antialias change
 
     moveToLayer((*m_rc_layernum).getNum());
 
-    // TODO: nls
-    m_layermenu.setLabel("Toolbar Layer");
-    m_placementmenu.setLabel("Toolbar Placement");
+    m_layermenu.setLabel(_FBTEXT(Toolbar, Layer, "Toolbar Layer", "Title of toolbar layer menu"));
+    m_placementmenu.setLabel(_FBTEXT(Toolbar, Placement, "Toolbar Placement", "Title of toolbar placement menu"));
 
     m_layermenu.setInternalMenu();
     m_placementmenu.setInternalMenu();
     setupMenus();
     // add menu to screen
-    screen().addConfigMenu("Toolbar", menu());
-    
+    screen().addConfigMenu(_FBTEXT(Toolbar, Toolbar, "Toolbar", "title of toolbar menu item"), menu());
+
     // geometry settings
     frame.width = width;
     frame.height = 10;
     frame.bevel_w = 1;
     frame.grab_x = frame.grab_y = 0;
-    
-    // set antialias on themes
-    m_tool_factory.updateThemes();
 
     // setup hide timer
     m_hide_timer.setTimeout(Fluxbox::instance()->getAutoRaiseDelay());
@@ -262,13 +264,14 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::XLayer &layer, size_t width):
 
     // show all windows
     frame.window.showSubwindows();
-    frame.window.show();
+    //    frame.window.show();
 
     scrn.resourceManager().unlock();
     // setup to listen to child events
     FbTk::EventManager::instance()->addParent(*this, window());
     // get everything together
-    reconfigure(); 
+    reconfigure();
+    // this gets done by the screen later as it loads
 
 }
 
@@ -301,7 +304,7 @@ void Toolbar::updateStrut() {
     // if the user dont want to request space for toolbar.
     if (doAutoHide() || *m_rc_maximize_over) {
         if (had_strut)
-            screen().updateAvailableWorkspaceArea();            
+            screen().updateAvailableWorkspaceArea();
         return;
     }
 
@@ -311,25 +314,25 @@ void Toolbar::updateStrut() {
     case TOPLEFT:
     case TOPCENTER:
     case TOPRIGHT:
-        top = height();
+        top = height() + 2 * theme().border().width();
         break;
     case BOTTOMLEFT:
     case BOTTOMCENTER:
     case BOTTOMRIGHT:
-        bottom = height();
+        bottom = height() + 2 * theme().border().width();
         break;
     case RIGHTTOP:
     case RIGHTCENTER:
     case RIGHTBOTTOM:
-        right = width();
+        right = width() + 2 * theme().border().width();
         break;
     case LEFTTOP:
     case LEFTCENTER:
     case LEFTBOTTOM:
-        left = width();
+        left = width() + 2 * theme().border().width();
         break;
     };
-    m_strut = screen().requestStrut(left, right, top, bottom);
+    m_strut = screen().requestStrut(getOnHead(), left, right, top, bottom);
     screen().updateAvailableWorkspaceArea();
 }
 
@@ -352,7 +355,7 @@ void Toolbar::lower() {
 }
 
 void Toolbar::reconfigure() {
-    updateVisibleState();
+    //    updateVisibleState();
 
     if (!doAutoHide() && isHidden())
         toggleHidden();
@@ -363,7 +366,7 @@ void Toolbar::reconfigure() {
 
     bool need_update = false;
     // parse and transform to lower case
-    std::list<std::string> tools;    
+    std::list<std::string> tools;
     FbTk::StringUtil::stringtok(tools, *m_rc_tools, ", ");
     transform(tools.begin(),
               tools.end(),
@@ -390,10 +393,10 @@ void Toolbar::reconfigure() {
         deleteItems();
 
         m_tools = tools; // copy values
-        
+
         if (m_tools.size()) {
             // make lower case
-            transform(m_tools.begin(), m_tools.end(), 
+            transform(m_tools.begin(), m_tools.end(),
                       m_tools.begin(),
                       FbTk::StringUtil::toLower);
 
@@ -405,6 +408,7 @@ void Toolbar::reconfigure() {
                 if (item == 0)
                     continue;
                 m_item_list.push_back(item);
+                item->resizeSig().attach(this);
 
             }
             // show all items
@@ -417,14 +421,14 @@ void Toolbar::reconfigure() {
 
     frame.bevel_w = theme().bevelWidth();
     // destroy shape if the theme wasn't specified with one,
-    // or create one 
+    // or create one
     if (theme().shape() == false && m_shape.get())
         m_shape.reset(0);
     else if (theme().shape() && m_shape.get() == 0) {
         m_shape.reset(new Shape(frame.window, 0));
     }
 
-    // recallibrate size
+    // recalibrate size
     setPlacement(placement());
 
     if (isHidden()) {
@@ -447,21 +451,35 @@ void Toolbar::reconfigure() {
     }
     if (tmp)
         screen().imageControl().removeImage(tmp);
-        
+
     frame.window.setBorderColor(theme().border().color());
     frame.window.setBorderWidth(theme().border().width());
-    frame.window.setAlpha(theme().alpha());
+
+    bool have_composite = FbTk::Transparent::haveComposite();
+    if (have_composite) {
+        frame.window.setOpaque(alpha());
+    } else {
+        frame.window.setAlpha(alpha());
+    }
     frame.window.clear();
-    frame.window.updateTransparent();
-    
+
     if (theme().shape() && m_shape.get())
         m_shape->update();
 
+    ItemList::iterator item_it = m_item_list.begin();
+    ItemList::iterator item_it_end = m_item_list.end();
+    for (; item_it != item_it_end; ++item_it) {
+        (*item_it)->updateSizing();
+    }
 
     rearrangeItems();
 
+    for (item_it = m_item_list.begin(); item_it != item_it_end; ++item_it) {
+        (*item_it)->renderTheme(alpha());
+    }
+
     menu().reconfigure();
-    // we're done with all resizing and stuff now we can request a new 
+    // we're done with all resizing and stuff now we can request a new
     // area to be reserved on screen
     updateStrut();
 
@@ -476,27 +494,25 @@ void Toolbar::buttonPressEvent(XButtonEvent &be) {
     screen().hideMenus();
 
     if (! menu().isVisible()) {
-        int x, y;
 
-        x = be.x_root - (menu().width() / 2);
-        y = be.y_root - (menu().height() / 2);
+        int head = screen().getHead(be.x_root, be.y_root);
+        int borderw = menu().fbwindow().borderWidth();
+        pair<int, int> m = screen().clampToHead(head,
+                                                be.x_root - (menu().width() / 2),
+                                                be.y_root - (menu().titleWindow().height() / 2),
+                                                menu().width() + 2*borderw,
+                                                menu().height() + 2*borderw);
 
-        if (x < 0)
-            x = 0;
-        else if (x + menu().width() > screen().width())
-            x = screen().width() - menu().width();
-
-        if (y < 0)
-            y = 0;
-        else if (y + menu().height() > screen().height())
-            y = screen().height() - menu().height();
-
-        menu().move(x, y);
+        menu().setScreen(screen().getHeadX(head),
+                         screen().getHeadY(head),
+                         screen().getHeadWidth(head),
+                         screen().getHeadHeight(head));
+        menu().move(m.first, m.second);
         menu().show();
         menu().grabInputFocus();
     } else
         menu().hide();
-	
+
 }
 
 
@@ -534,9 +550,9 @@ void Toolbar::leaveNotifyEvent(XCrossingEvent &event) {
         return;
 
     if (isHidden()) {
-        if (m_hide_timer.isTiming()) 
+        if (m_hide_timer.isTiming())
             m_hide_timer.stop();
-    } else if (! menu().isVisible() && ! m_hide_timer.isTiming()) 
+    } else if (! menu().isVisible() && ! m_hide_timer.isTiming())
         m_hide_timer.start();
 
 }
@@ -546,23 +562,32 @@ void Toolbar::exposeEvent(XExposeEvent &ee) {
     if (ee.window == frame.window) {
         frame.window.clearArea(ee.x, ee.y,
                                ee.width, ee.height);
-        frame.window.updateTransparent(ee.x, ee.y,
-                               ee.width, ee.height);
     }
 }
 
 
 void Toolbar::handleEvent(XEvent &event) {
+    /* Commented out by Simon 16jun04, since it causes LOTS of rearrangeItems
+       particularly on startup. This was needed to resize when tool changes its own
+       size, but it has too many side effects. Use the resizeSig in ToolbarItem instead.
+
     if (event.type == ConfigureNotify &&
-        event.xconfigure.window != window().window())
-            rearrangeItems();
+        event.xconfigure.window != window().window()) {
+        rearrangeItems();
+    }
+*/
 }
 
 void Toolbar::update(FbTk::Subject *subj) {
 
-    // either screen reconfigured or theme was reloaded
-    
-    reconfigure();
+    // either screen reconfigured, theme was reloaded
+    // or a tool resized itself
+
+    if (typeid(*subj) == typeid(ToolbarItem::ToolbarItemSubject))
+        rearrangeItems();
+    else
+        reconfigure();
+
 }
 
 void Toolbar::setPlacement(Toolbar::Placement where) {
@@ -594,8 +619,11 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
         head_h = screen().getHeadHeight(head);
     }
 
-    frame.width = head_w * (*m_rc_width_percent) / 100;
-    //!! TODO: change this 
+    int bevel_width = theme().bevelWidth();
+    int border_width = theme().border().width();
+
+    frame.width = (head_w - 2*border_width) * (*m_rc_width_percent) / 100;
+    //!! TODO: change this
     // max height of each toolbar items font...
     unsigned int max_height = m_tool_factory.maxFontHeight();
 
@@ -610,9 +638,6 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
     frame.height += 2;
     frame.height += (frame.bevel_w * 2);
 
-    int bevel_width = theme().bevelWidth();
-    int border_width = theme().border().width();
-
     // should we flipp sizes?
     if (isVertical()) {
         frame.width = frame.height;
@@ -645,7 +670,7 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
         break;
 
     case TOPCENTER:
-        frame.x = head_x + (head_w - frame.width) / 2;
+        frame.x = head_x + (head_w - frame.width) / 2 - border_width;
         frame.y = head_y;
         frame.x_hidden = frame.x;
         frame.y_hidden = head_y + bevel_width - border_width - frame.height;
@@ -656,6 +681,7 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
         frame.x = head_x + head_w - frame.width - border_width*2;
         frame.y = head_y;
         frame.x_hidden = frame.x;
+        frame.y_hidden = head_y + bevel_width - border_width - frame.height;
         if (m_shape.get())
             m_shape->setPlaces(Shape::BOTTOMRIGHT | Shape::BOTTOMLEFT);
         break;
@@ -671,7 +697,7 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
 
     case BOTTOMCENTER: // default is BOTTOMCENTER
     default:
-        frame.x = head_x + (head_w - frame.width) / 2;
+        frame.x = head_x + (head_w - frame.width) / 2 - border_width;
         frame.y = head_y + head_h - frame.height - border_width*2;
         frame.x_hidden = frame.x;
         frame.y_hidden = head_y + head_h - bevel_width - border_width;
@@ -680,7 +706,7 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
         break;
     case LEFTCENTER:
         frame.x = head_x;
-        frame.y = head_y + (head_h - frame.height)/2;
+        frame.y = head_y + (head_h - frame.height)/2 - border_width;
         frame.x_hidden = frame.x - frame.width + bevel_width + border_width;
         frame.y_hidden = frame.y;
         if (m_shape.get())
@@ -704,7 +730,7 @@ void Toolbar::setPlacement(Toolbar::Placement where) {
         break;
     case RIGHTCENTER:
         frame.x = head_x + head_w - frame.width - border_width*2;
-        frame.y = head_y + (head_h - frame.height)/2;
+        frame.y = head_y + (head_h - frame.height)/2 - border_width;
         frame.x_hidden = frame.x + frame.width - bevel_width - border_width;
         frame.y_hidden = frame.y;
         if (m_shape.get())
@@ -740,38 +766,37 @@ void Toolbar::toggleHidden() {
     m_hidden = ! m_hidden;
     if (isHidden())
         frame.window.move(frame.x_hidden, frame.y_hidden);
-    else 
+    else
         frame.window.move(frame.x, frame.y);
 
 }
 
 void Toolbar::moveToLayer(int layernum) {
-    m_layeritem.moveToLayer(layernum); 
+    m_layeritem.moveToLayer(layernum);
     *m_rc_layernum = layernum;
 }
 
 void Toolbar::setupMenus() {
-    const I18n &i18n = *I18n::instance();
-    using namespace FBNLS;
+    _FB_USES_NLS;
     using namespace FbTk;
 
     typedef RefCount<Command> RefCommand;
     typedef SimpleCommand<Toolbar> ToolbarCommand;
 
     //!! TODO: this should be inserted by the workspace tool
-        
+
 
     RefCommand start_edit(CommandParser::instance().parseLine("setworkspacenamedialog"));
-    menu().insert(i18n.getMessage(FBNLS::ToolbarSet, FBNLS::ToolbarEditWkspcName,
-                                   "Edit current workspace name"),
+    menu().insert(_FBTEXT(Toolbar, EditWkspcName,
+                          "Edit current workspace name", "Edit current workspace name"),
                   start_edit);
-    
-    menu().setLabel(i18n.getMessage(FBNLS::ToolbarSet, FBNLS::ToolbarToolbarTitle,
-                                     "Toolbar")); 
 
-    MenuItem *toolbar_menuitem = new IntResMenuItem("Toolbar width percent",
+    menu().setLabel(_FBTEXT(Toolbar, Toolbar,
+                            "Toolbar", "Title of Toolbar menu"));
+
+    MenuItem *toolbar_menuitem = new IntResMenuItem(_FBTEXT(Toolbar, WidthPercent, "Toolbar width percent", "Percentage of screen width taken by toolbar"),
                                                     m_rc_width_percent,
-                                                    0, 100); // min/max value
+                                                    0, 100, menu()); // min/max value
 
 
     RefCommand reconfig_toolbar(new ToolbarCommand(*this, &Toolbar::reconfigure));
@@ -781,12 +806,12 @@ void Toolbar::setupMenus() {
     toolbar_menuitem_macro->add(save_resources);
 
     RefCommand reconfig_toolbar_and_save_resource(toolbar_menuitem_macro);
-    toolbar_menuitem->setCommand(reconfig_toolbar_and_save_resource);  
+    toolbar_menuitem->setCommand(reconfig_toolbar_and_save_resource);
 
     menu().insert(toolbar_menuitem);
 
-    menu().insert(new BoolMenuItem(i18n.getMessage(FBNLS::CommonSet, FBNLS::CommonAutoHide,
-                                                    "Auto hide"),
+    menu().insert(new BoolMenuItem(_FBTEXT(Common, AutoHide,
+                                           "Auto hide", "Toggle auto hide of toolbar"),
                                    *m_rc_auto_hide,
                                    reconfig_toolbar_and_save_resource));
 
@@ -795,68 +820,83 @@ void Toolbar::setupMenus() {
     visible_macro->add(toggle_visible);
     visible_macro->add(save_resources);
     RefCommand toggle_visible_cmd(visible_macro);
-    menu().insert(new BoolMenuItem("Visible", *m_rc_visible, toggle_visible_cmd));
+    menu().insert(new BoolMenuItem(_FBTEXT(Common, Visible, "Visible", "Whether this item is visible"),
+                                   *m_rc_visible, toggle_visible_cmd));
 
-    menu().insert(new BoolMenuItem("Maximize Over", *m_rc_maximize_over,
+    menu().insert(new BoolMenuItem(_FBTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
+                                   *m_rc_maximize_over,
                                    reconfig_toolbar_and_save_resource));
-    menu().insert("Layer...", &layermenu());
+    menu().insert(_FBTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), &layermenu());
 
 
 
     if (screen().hasXinerama()) {
-        // TODO: nls (main label plus menu heading
-        menu().insert("On Head...", new XineramaHeadMenu<Toolbar>(screen().menuTheme(),
-                                                                  screen(),
-                                                                  screen().imageControl(),
-                                                                  *screen().layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
-                                                                  *this,
-                                                                  "Toolbar on Head"));
+        menu().insert(_FBTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
+                      new XineramaHeadMenu<Toolbar>(screen().menuTheme(),
+                                                    screen(),
+                                                    screen().imageControl(),
+                                                    *screen().layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
+                                                    *this,
+                                                    _FBTEXT(Toolbar, OnHead, "Toolbar on Head", "Title of toolbar on head menu")));
     }
 
-    // setup items in placement menu
-    struct {
-        int set;
-        int base;
-        const char *default_str;
-        Toolbar::Placement placement;
-    } place_menu[]  = {
-        {0, 0, "Top Left", Toolbar::TOPLEFT},
-        
-        {0, 0, "Left Top", Toolbar::LEFTTOP},
-        {0, 0, "Left Center", Toolbar::LEFTCENTER},
-        {0, 0, "Left Bottom", Toolbar::LEFTBOTTOM}, 
-        
-        {0, 0, "Bottom Left", Toolbar::BOTTOMLEFT},
-        {0, 0, "Top Center", Toolbar::TOPCENTER},
-        {0, 0, 0, Toolbar::TOPCENTER},
-        {0, 0, 0, Toolbar::BOTTOMCENTER},
-        {0, 0, 0, Toolbar::BOTTOMCENTER},
-        {0, 0, "Bottom Center", Toolbar::BOTTOMCENTER},
-        {0, 0, "Top Right", Toolbar::TOPRIGHT},
-        
-        {0, 0, "Right Top", Toolbar::RIGHTTOP},
-        {0, 0, "Right Center", Toolbar::RIGHTCENTER},
-        {0, 0, "Right Bottom", Toolbar::RIGHTBOTTOM},
-        
-        {0, 0, "Bottom Right", Toolbar::BOTTOMRIGHT}
-    };
+    typedef pair<const char*, Toolbar::Placement> PlacementP;
+    typedef list<PlacementP> Placements;
+    Placements place_menu;
+
+    // menu is 3 wide, 5 down
+    place_menu.push_back(PlacementP(_FBTEXT(Align, TopLeft, "Top Left", "Top Left"), Toolbar::TOPLEFT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, LeftTop, "Left Top", "Left Top"), Toolbar::LEFTTOP));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, LeftCenter, "Left Center", "Left Center"), Toolbar::LEFTCENTER));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, LeftBottom, "Left Bottom", "Left Bottom"), Toolbar::LEFTBOTTOM));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomLeft, "Bottom Left", "Bottom Left"), Toolbar::BOTTOMLEFT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, TopCenter, "Top Center", "Top Center"), Toolbar::TOPCENTER));
+    place_menu.push_back(PlacementP((const char *)0, Toolbar::TOPLEFT));
+    place_menu.push_back(PlacementP((const char *)0, Toolbar::TOPLEFT));
+    place_menu.push_back(PlacementP((const char *)0, Toolbar::TOPLEFT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomCenter, "Bottom Center", "Bottom Center"), Toolbar::BOTTOMCENTER));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, TopRight, "Top Right", "Top Right"), Toolbar::TOPRIGHT));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, RightTop, "Right Top", "Right Top"), Toolbar::RIGHTTOP));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, RightCenter, "Right Center", "Right Center"), Toolbar::RIGHTCENTER));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, RightBottom, "Right Bottom", "Right Bottom"), Toolbar::RIGHTBOTTOM));
+    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomRight, "Bottom Right", "Bottom Right"), Toolbar::BOTTOMRIGHT));
+
+
     placementMenu().setMinimumSublevels(3);
     // create items in sub menu
     for (size_t i=0; i<15; ++i) {
-        if (place_menu[i].default_str == 0) {
+        const char *str = place_menu.front().first;
+        Toolbar::Placement placement = place_menu.front().second;
+
+        if (str == 0) {
             placementMenu().insert("");
+            placementMenu().setItemEnabled(i, false);
         } else {
-            const char *i18n_str = i18n.getMessage(place_menu[i].set, 
-                                                    place_menu[i].base,
-                                                    place_menu[i].default_str);
-            RefCommand setplace(new SetToolbarPlacementCmd(*this, place_menu[i].placement));
-            placementMenu().insert(i18n_str, setplace);
-                                                              
+            RefCommand setplace(new SetToolbarPlacementCmd(*this, placement));
+            placementMenu().insert(str, setplace);
+
         }
+        place_menu.pop_front();
     }
-    menu().insert("Placement", &placementMenu());
-    placementMenu().update();
-    menu().update();
+    menu().insert(_FBTEXT(Menu, Placement, "Placement", "Title of Placement menu"), &placementMenu());
+    placementMenu().updateMenu();
+
+
+    // this saves resources and clears the slit window to update alpha value
+    FbTk::MenuItem *alpha_menuitem =
+        new IntResMenuItem(_FBTEXT(Common, Alpha, "Alpha", "Transparency level"),
+                           m_rc_alpha,
+                           0, 255, menu());
+    // setup command for alpha value
+    MacroCommand *alpha_macrocmd = new MacroCommand();
+    RefCount<Command> alpha_cmd(new SimpleCommand<Toolbar>(*this, &Toolbar::updateAlpha));
+    alpha_macrocmd->add(save_resources);
+    alpha_macrocmd->add(alpha_cmd);
+    RefCount<Command> set_alpha_cmd(alpha_macrocmd);
+    alpha_menuitem->setCommand(set_alpha_cmd);
+
+    menu().insert(alpha_menuitem);
+    menu().updateMenu();
 }
 
 void Toolbar::saveOnHead(int head) {
@@ -864,23 +904,58 @@ void Toolbar::saveOnHead(int head) {
     reconfigure();
 }
 
+/*
+ * Place items next to each other, with a bevel width between,
+ * above and below each item. BUT, if there is no bevel width, then
+ * borders should be merged for evenness.
+ */
+
 void Toolbar::rearrangeItems() {
     if (m_resize_lock || screen().isShuttingdown() ||
         m_item_list.empty())
         return;
+
     // lock this
     m_resize_lock = true;
     // calculate size for fixed items
     ItemList::iterator item_it = m_item_list.begin();
     ItemList::iterator item_it_end = m_item_list.end();
-    int fixed_width = 0; // combined size of all fixed items
+    int bevel_width = theme().bevelWidth();
+    int fixed_width = bevel_width; // combined size of all fixed items
     int fixed_items = 0; // number of fixed items
     int relative_items = 0;
+    int last_bw = 0; // we show the largest border of adjoining items
+    bool first = true;
     for (; item_it != item_it_end; ++item_it) {
-        if ((*item_it)->type() == ToolbarItem::FIXED && (*item_it)->active()) {
-            fixed_width += (*item_it)->width() + (*item_it)->borderWidth()*2;
+        if (!(*item_it)->active())
+            continue;
+
+        int borderW = (*item_it)->borderWidth();
+
+        if (bevel_width > 0) {
+            // the bevel and border are fixed whether relative or not
+            fixed_width += bevel_width + 2*borderW;
+        } else {
+            if (!first) {
+                if (borderW > last_bw)
+                    fixed_width += borderW;
+                else
+                    fixed_width += last_bw;
+            } else {
+                first = false;
+            }
+        }
+
+        last_bw = borderW;
+
+        if ((*item_it)->type() == ToolbarItem::FIXED) {
+            fixed_width += (*item_it)->width();
             fixed_items++;
-        } else if ((*item_it)->type() == ToolbarItem::RELATIVE && (*item_it)->active()) {
+        } else if ((*item_it)->type() == ToolbarItem::SQUARE) {
+            fixed_width += height() - 2*bevel_width;
+            if (bevel_width != 0) fixed_width -= 2*borderW;
+            fixed_items++;
+        } else {
             relative_items++;
         }
     }
@@ -895,38 +970,61 @@ void Toolbar::rearrangeItems() {
             relative_width = 0;
         else { // size left after fixed items / number of relative items
             relative_width = (width() - fixed_width)/relative_items;
-            rounding_error = width() - fixed_width - relative_items*relative_width;
+            rounding_error = width() - fixed_width - relative_items*(relative_width);
         }
     }
+
     // now move and resize the items
-    int next_x = 0;
+    // borderWidth added back on straight away
+    int next_x = -m_item_list.front()->borderWidth(); // list isn't empty
+    if (bevel_width != 0)
+        next_x = 0;
+
+    last_bw = 0;
     for (item_it = m_item_list.begin(); item_it != item_it_end; ++item_it) {
+        int borderW = (*item_it)->borderWidth();
         if (!(*item_it)->active()) {
             (*item_it)->hide();
+            // make sure it still gets told the toolbar height
+            (*item_it)->resize(1, height()-2*(bevel_width+borderW));  // width of 0 changes to 1 anyway
             continue;
         }
+        int offset = bevel_width;
+        int size_offset = 2*(borderW + bevel_width);
+
+        if (bevel_width == 0) {
+            offset = -borderW;
+            size_offset = 0;
+            if (borderW > last_bw)
+                next_x += borderW;
+            else
+                next_x += last_bw;
+        }
+        last_bw = borderW;
 
-        int borderW = (*item_it)->borderWidth();
-        (*item_it)->show();
         if ((*item_it)->type() == ToolbarItem::RELATIVE) {
             int extra = 0;
             if (rounding_error != 0) { // distribute rounding error over all relatives
                 extra = 1;
                 --rounding_error;
             }
-
-            (*item_it)->moveResize(next_x, -borderW, extra + relative_width-2*borderW, height());
+            (*item_it)->moveResize(next_x + offset, offset, extra + relative_width, height() - size_offset);
+        } else if ((*item_it)->type() == ToolbarItem::SQUARE) {
+            (*item_it)->moveResize(next_x + offset, offset,
+                                   height() - size_offset, height() - size_offset);
         } else { // fixed size
-            (*item_it)->moveResize(next_x, -borderW,
-                                   (*item_it)->width(), height()); 
+            (*item_it)->moveResize(next_x + offset, offset,
+                                   (*item_it)->width(), height() - size_offset);
         }
-        next_x += (*item_it)->width() + borderW*2;
+        (*item_it)->show();
+        next_x += (*item_it)->width() + bevel_width;
+        if (bevel_width != 0)
+            next_x += 2*borderW;
+
     }
     // unlock
     m_resize_lock = false;
     frame.window.clear();
-    frame.window.updateTransparent();
-
 }
 
 void Toolbar::deleteItems() {
@@ -936,3 +1034,19 @@ void Toolbar::deleteItems() {
     }
     m_tools.clear();
 }
+
+void Toolbar::updateAlpha() {
+    // called when the alpha resource is changed
+    if (FbTk::Transparent::haveComposite()) {
+        frame.window.setOpaque(*m_rc_alpha);
+    } else {
+        frame.window.setAlpha(*m_rc_alpha);
+        frame.window.clear();
+
+        ItemList::iterator item_it = m_item_list.begin();
+        ItemList::iterator item_it_end = m_item_list.end();
+        for (item_it = m_item_list.begin(); item_it != item_it_end; ++item_it) {
+            (*item_it)->renderTheme(alpha());
+        }
+    }
+}
