@@ -1,5 +1,5 @@
 // ToolbarTheme.cc  a theme class for Toolbar
-// Copyright (c) 2002-2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ToolbarTheme.cc,v 1.13 2003/10/13 23:47:38 fluxgen Exp $
+// $Id: ToolbarTheme.cc 3865 2005-01-24 18:34:57Z mathias $
 
 #include "ToolbarTheme.hh"
 
@@ -28,34 +28,16 @@
 #include <iostream>
 using namespace std;
 
-template<>
-void FbTk::ThemeItem<bool>::load() { }
-
-template<>
-void FbTk::ThemeItem<bool>::setDefaultValue() {
-    *(*this) = false;
-}
-
-template<>
-void FbTk::ThemeItem<bool>::setFromString(char const *strval) {
-    if (strcasecmp(strval, "true")==0)
-        *(*this) = true;
-    else
-        *(*this) = false;
-}
-
 ToolbarTheme::ToolbarTheme(int screen_num):
     FbTk::Theme(screen_num),
     m_toolbar(*this, "toolbar", "Toolbar"),
     m_border(*this, "toolbar", "Toolbar"),
     m_bevel_width(*this, "toolbar.bevelWidth", "Toolbar.BevelWidth"),
     m_shape(*this, "toolbar.shaped", "Toolbar.Shaped"),    
-    m_alpha(*this, "toolbar.alpha", "Toolbar.Alpha"),
     m_height(*this, "toolbar.height", "Toolbar.Height"),
     m_button_size(*this, "toolbar.button.size", "Toolbar.Button.Size") {
     // set default value
     *m_bevel_width = 0;
-    *m_alpha = 255;
     *m_shape = false;
     *m_height = 0;
     *m_button_size = -1;
@@ -65,12 +47,18 @@ ToolbarTheme::~ToolbarTheme() {
 
 }
 
-void ToolbarTheme::reconfigTheme() {
-    if (*m_alpha > 255)
-        *m_alpha = 255;
-    else if (*m_alpha < 0)
-        *m_alpha = 0;
+bool ToolbarTheme::fallback(FbTk::ThemeItem_base &item) {
+    if (item.name().find(".borderWidth") != std::string::npos) {
+        return FbTk::ThemeManager::instance().loadItem(item, "borderWidth", "BorderWidth");
+    } else if (item.name().find(".borderColor") != std::string::npos) {
+        return FbTk::ThemeManager::instance().loadItem(item, "borderColor", "BorderColor");
+    } else if (item.name() == "toolbar.bevelWidth") {
+        return FbTk::ThemeManager::instance().loadItem(item, "bevelWidth", "BevelWidth");
+    }
+    return false;
+}
 
+void ToolbarTheme::reconfigTheme() {
     if (*m_bevel_width > 20)
         *m_bevel_width = 20;
 
