@@ -1,5 +1,5 @@
 // Workspace.cc for Fluxbox
-// Copyright (c) 2001 - 2004 Henrik Kinnunen (fluxgen[at]users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Workspace.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Workspace.cc 3770 2004-12-13 14:03:17Z mathias $
+// $Id: Workspace.cc 3957 2005-04-27 09:52:30Z mathias $
 
 #include "Workspace.hh"
 
@@ -92,9 +92,15 @@ public:
             return;
         FluxboxWindow &win = *m_client.fbwindow();
 
-        win.screen().changeWorkspaceID(win.workspaceNumber()); 
-        win.setCurrentClient(m_client);
-        win.raiseAndFocus();
+        // fetch the window to the current workspace
+        if (button == 2 && win.screen().currentWorkspaceID() != win.workspaceNumber()) {
+            win.menu().hide();
+            win.screen().sendToWorkspace(win.screen().currentWorkspaceID(), &win, true);
+        } else { // warp to the workspace of the window
+            win.screen().changeWorkspaceID(win.workspaceNumber()); 
+            win.setCurrentClient(m_client);
+            win.raiseAndFocus();
+        }
     }
 
     const std::string &label() const { return m_client.title(); }
@@ -358,7 +364,7 @@ void Workspace::update(FbTk::Subject *subj) {
 
 
 void Workspace::setName(const std::string &name) {
-    if (!name.empty()) {
+    if (!name.empty() && name != "") {
         m_name = name;
     } else { //if name == 0 then set default name from nls
         _FB_USES_NLS;
@@ -369,7 +375,7 @@ void Workspace::setName(const std::string &name) {
                 m_id + 1); //m_id starts at 0
         m_name = tname;
     }
-	
+    
     screen().updateWorkspaceNamesAtom();
 	
     menu().setLabel(m_name.c_str());
