@@ -1,5 +1,5 @@
 // WorkspaceNameTool.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WorkspaceNameTool.cc,v 1.7 2004/01/13 14:41:32 rathnor Exp $
+// $Id: WorkspaceNameTool.cc 3957 2005-04-27 09:52:30Z mathias $
 
 #include "WorkspaceNameTool.hh"
 
@@ -30,6 +30,8 @@
 
 #include "FbTk/ImageControl.hh"
 
+#include <algorithm>
+
 WorkspaceNameTool::WorkspaceNameTool(const FbTk::FbWindow &parent, 
                                      ToolTheme &theme, BScreen &screen):
     ToolbarItem(ToolbarItem::FIXED),
@@ -46,7 +48,6 @@ WorkspaceNameTool::WorkspaceNameTool(const FbTk::FbWindow &parent,
     screen.currentWorkspaceSig().attach(this);
     theme.reconfigSig().attach(this);
 
-    renderTheme();
 }
 
 WorkspaceNameTool::~WorkspaceNameTool() {
@@ -69,21 +70,25 @@ void WorkspaceNameTool::moveResize(int x, int y,
 }
 
 void WorkspaceNameTool::update(FbTk::Subject *subj) {
+
     m_button.setText(m_screen.currentWorkspace()->name());
-    if (m_button.width() != width())
+    if (m_button.width() != width()) {
         resize(width(), height());
-    renderTheme();
+        resizeSig().notify();
+    }
+    reRender();
+    m_button.clear();
 }
 
 unsigned int WorkspaceNameTool::width() const {
     // calculate largest size
-    int max_size = 0;
-    const int num_workspaces = m_screen.getNumberOfWorkspaces();
-    for (int workspace = 0; workspace < num_workspaces; ++workspace) {
-        const std::string &name = m_screen.getWorkspace(workspace)->name().c_str();
-        int size = m_theme.font().textWidth(name.c_str(), name.size());
-        if (size > max_size)
-            max_size = size;
+    unsigned int max_size = 0;
+    const BScreen::Workspaces& workspaces = m_screen.getWorkspacesList();
+    BScreen::Workspaces::const_iterator it;
+    for (it = workspaces.begin(); it != workspaces.end(); it++) {
+        const std::string &name = (*it)->name();
+        max_size = std::max(m_theme.font().textWidth(name.c_str(), name.size()), 
+                            max_size);
     }
     // so align text dont cut the last character
     max_size += 2;
@@ -107,23 +112,33 @@ void WorkspaceNameTool::hide() {
     m_button.hide();
 }
 
-void WorkspaceNameTool::renderTheme() {
-    Pixmap tmp = m_pixmap;
-    if (!m_theme.texture().usePixmap()) {
-        m_pixmap = 0;
-        m_button.setBackgroundColor(m_theme.texture().color());
-    } else {
+void WorkspaceNameTool::updateSizing() {
+    m_button.setBorderWidth(m_theme.border().width());
+}
+
+void WorkspaceNameTool::reRender() {
+    if (m_theme.texture().usePixmap()) {
+        if (m_pixmap) 
+            m_screen.imageControl().removeImage(m_pixmap);
         m_pixmap = m_screen.imageControl().renderImage(width(), height(),
                                                        m_theme.texture());
         m_button.setBackgroundPixmap(m_pixmap);
     }
-    if (tmp)
-        m_screen.imageControl().removeImage(tmp);
+}
+
+void WorkspaceNameTool::renderTheme(unsigned char alpha) {
+    if (!m_theme.texture().usePixmap()) {
+        if (m_pixmap)
+            m_screen.imageControl().removeImage(m_pixmap);
+        m_pixmap = 0;
+        m_button.setBackgroundColor(m_theme.texture().color());
+    } else {
+        reRender();
+    }
 
     m_button.setJustify(m_theme.justify());
     m_button.setBorderWidth(m_theme.border().width());
     m_button.setBorderColor(m_theme.border().color());
-    m_button.setAlpha(m_theme.alpha());
+    m_button.setAlpha(alpha);
     m_button.clear();
-    m_button.updateTransparent();
 }
