@@ -1,5 +1,5 @@
 // fluxbox.cc for Fluxbox Window Manager
-// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2001 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // blackbox.cc for blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: fluxbox.cc 3835 2005-01-14 09:40:40Z mathias $
+// $Id: fluxbox.cc 4004 2005-05-10 16:29:00Z simonb $
 
 #include "fluxbox.hh"
 
@@ -51,6 +51,8 @@
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/CompareEqual.hh"
 #include "FbTk/Transparent.hh"
+#include "FbTk/Select2nd.hh"
+#include "FbTk/Compose.hh"
 
 //Use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -156,6 +158,8 @@ namespace {
 
 Window last_bad_window = None;
  
+// *** NOTE: if you want to debug here the X errors are 
+//     coming from, you should turn on the XSynchronise call below
 int handleXErrors(Display *d, XErrorEvent *e) {
     if (e->error_code == BadWindow)
         last_bad_window = e->resourceid;
@@ -200,7 +204,6 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
                               "session.colorsPerChannel", "Session.ColorsPerChannel"),
       m_rc_numlayers(m_resourcemanager, 13, "session.numLayers", "Session.NumLayers"),
       m_rc_double_click_interval(m_resourcemanager, 250, "session.doubleClickInterval", "Session.DoubleClickInterval"),
-      m_rc_update_delay_time(m_resourcemanager, 0, "session.updateDelayTime", "Session.UpdateDelayTime"),
       m_rc_stylefile(m_resourcemanager, DEFAULTSTYLE, "session.styleFile", "Session.StyleFile"),
       m_rc_menufile(m_resourcemanager, DEFAULTMENU, "session.menuFile", "Session.MenuFile"),
       m_rc_keyfile(m_resourcemanager, DEFAULTKEYSFILE, "session.keyFile", "Session.KeyFile"),
@@ -278,7 +281,7 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
     m_reconfig_timer.setTimeout(to);
     m_reconfig_timer.setCommand(reconfig_cmd);
     m_reconfig_timer.fireOnce(true);
-    //XSynchronize(disp, True);
+    // XSynchronize(disp, True);
 
     s_singleton = this;
     m_have_shape = false;
@@ -324,92 +327,61 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
     m_fluxbox_pid = XInternAtom(disp, "_BLACKBOX_PID", False);
 #endif // HAVE_GETPID
 
-    // Allocate screens
-    for (int i = 0; i < ScreenCount(display()); i++) {
-        char scrname[128], altscrname[128];
-        sprintf(scrname, "session.screen%d", i);
-        sprintf(altscrname, "session.Screen%d", i);
-        BScreen *screen = new BScreen(m_screen_rm.lock(),
-                                      scrname, altscrname,
-                                      i, getNumberOfLayers());
-        if (! screen->isScreenManaged()) {
-            delete screen;
-            continue;
+
+    vector<int> screens;
+    int i;
+
+    // default is "use all screens"
+    for (i = 0; i < ScreenCount(disp); i++)
+        screens.push_back(i);
+
+    // find out, on what "screens" fluxbox should run
+    for (i = 1; i < m_argc; i++) {
+        if (! strcmp(m_argv[i], "-screen")) {
+            if ((++i) >= m_argc) {
+                cerr<<"error, -screen requires argument\n";
+                exit(1);
+            }
+
+            // "all" is default
+            if (! strcmp(m_argv[i], "all"))
+                break;
+
+            vector<string> vals;
+            vector<int> scrtmp;
+            int scrnr = 0;
+            FbTk::StringUtil::stringtok(vals, m_argv[i], ",:");
+            for (vector<string>::iterator scrit = vals.begin(); 
+                 scrit != vals.end(); scrit++) {
+                scrnr = atoi(scrit->c_str());
+                if (scrnr >= 0 && scrnr < ScreenCount(disp))
+                    scrtmp.push_back(scrnr);
+            }
+
+            if (!vals.empty())
+                swap(scrtmp, screens);
         }
-        // add to our list
-        m_screen_list.push_back(screen);
-
-        // now we can create menus (which needs this screen to be in screen_list)
-        screen->initMenus();
-
-#ifdef HAVE_GETPID
-        pid_t bpid = getpid();
-
-        screen->rootWindow().changeProperty(getFluxboxPidAtom(), XA_CARDINAL,
-                                            sizeof(pid_t) * 8, PropModeReplace,
-                                            (unsigned char *) &bpid, 1);
-#endif // HAVE_GETPID
-
-#ifdef HAVE_RANDR
-        // setup RANDR for this screens root window
-        // we need to determine if we should use old randr select input function or not
-#ifdef X_RRScreenChangeSelectInput
-        // use old set randr event
-        XRRScreenChangeSelectInput(disp, screen->rootWindow().window(), True);
-#else
-        XRRSelectInput(disp, screen->rootWindow().window(),
-                       RRScreenChangeNotifyMask);
-#endif // X_RRScreenChangeSelectInput
-
-#endif // HAVE_RANDR
-
-
-#ifdef USE_TOOLBAR
-        m_toolbars.push_back(new Toolbar(*screen,
-                                         *screen->layerManager().
-                                         getLayer(Fluxbox::instance()->getNormalLayer())));
-#endif // USE_TOOLBAR
-
-        // must do this after toolbar is created
-        screen->initWindows();
-
-        // attach screen signals to this
-        screen->currentWorkspaceSig().attach(this);
-        screen->workspaceCountSig().attach(this);
-        screen->workspaceNamesSig().attach(this);
-        screen->workspaceAreaSig().attach(this);
-        screen->clientListSig().attach(this);
-
-        // initiate atomhandler for screen specific stuff
-        for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end();
-             it++) {
-            (*it).first->initForScreen(*screen);
-        }
-
-        revertFocus(*screen); // make sure focus style is correct
-#ifdef SLIT
-        if (screen->slit())
-            screen->slit()->show();
-#endif // SLIT
-
-
-    } // end init screens
+    }
+    
+    // init all "screens"
+    for(i = 0; i < screens.size(); i++)
+        initScreen(screens[i]);
+    
     XAllowEvents(disp, ReplayPointer, CurrentTime);
 
 
-    m_keyscreen = m_mousescreen = m_screen_list.front();
-
     if (m_screen_list.empty()) {
         throw string(_FBTEXT(Fluxbox, ErrorNoScreens,
-                             "Couldn't find screens to manage.\nMake sure you don't have another window manager running.", "Error message when no unmanaged screens found - usually means another window manager is running"));
+                             "Couldn't find screens to manage.\nMake sure you don't have another window manager running.", 
+                             "Error message when no unmanaged screens found - usually means another window manager is running"));
     }
 
+    m_keyscreen = m_mousescreen = m_screen_list.front();
+   
     // setup theme manager to have our style file ready to be scanned
     FbTk::ThemeManager::instance().load(FbTk::StringUtil::expandFilename(getStyleFilename()));
 
-    XSynchronize(disp, False);
-    //XSynchronize(disp, True);
+    //XSynchronize(disp, False);
     sync(false);
 
     m_reconfigure_wait = m_reread_menu_wait = false;
@@ -419,7 +391,7 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
 
     m_resourcemanager.unlock();
     ungrab();
-
+ 
 #ifdef DEBUG
     if (m_resourcemanager.lockDepth() != 0)
         cerr<<"--- resource manager lockdepth = "<<m_resourcemanager.lockDepth()<<endl;
@@ -467,6 +439,84 @@ Fluxbox::~Fluxbox() {
     clearMenuFilenames();
 }
 
+
+int Fluxbox::initScreen(int scrnr) {
+    
+    Display* disp = display();
+    char scrname[128], altscrname[128];
+    sprintf(scrname, "session.screen%d", scrnr);
+    sprintf(altscrname, "session.Screen%d", scrnr);
+    BScreen *screen = new BScreen(m_screen_rm.lock(),
+                                  scrname, altscrname,
+                                  scrnr, getNumberOfLayers());
+
+    // already handled
+    if (! screen->isScreenManaged()) {
+        delete screen;
+        return 0;
+    }
+    
+    // add to our list
+    m_screen_list.push_back(screen);
+
+    // now we can create menus (which needs this screen to be in screen_list)
+    screen->initMenus();
+
+#ifdef HAVE_GETPID
+    pid_t bpid = getpid();
+
+    screen->rootWindow().changeProperty(getFluxboxPidAtom(), XA_CARDINAL,
+                                        sizeof(pid_t) * 8, PropModeReplace,
+                                        (unsigned char *) &bpid, 1);
+#endif // HAVE_GETPID
+
+#ifdef HAVE_RANDR
+    // setup RANDR for this screens root window
+    // we need to determine if we should use old randr select input function or not
+#ifdef X_RRScreenChangeSelectInput
+    // use old set randr event
+    XRRScreenChangeSelectInput(disp, screen->rootWindow().window(), True);
+#else
+    XRRSelectInput(disp, screen->rootWindow().window(),
+                   RRScreenChangeNotifyMask);
+#endif // X_RRScreenChangeSelectInput
+
+#endif // HAVE_RANDR
+
+
+#ifdef USE_TOOLBAR
+    m_toolbars.push_back(new Toolbar(*screen,
+                                     *screen->layerManager().
+                                     getLayer(Fluxbox::instance()->getNormalLayer())));
+#endif // USE_TOOLBAR
+
+    // must do this after toolbar is created
+    screen->initWindows();
+
+    // attach screen signals to this
+    screen->currentWorkspaceSig().attach(this);
+    screen->workspaceCountSig().attach(this);
+    screen->workspaceNamesSig().attach(this);
+    screen->workspaceAreaSig().attach(this);
+    screen->clientListSig().attach(this);
+
+    // initiate atomhandler for screen specific stuff
+    for (AtomHandlerContainerIt it= m_atomhandler.begin();
+         it != m_atomhandler.end();
+         it++) {
+        (*it).first->initForScreen(*screen);
+    }
+
+    revertFocus(*screen); // make sure focus style is correct
+#ifdef SLIT
+    if (screen->slit())
+        screen->slit()->show();
+#endif // SLIT
+
+    return 1;
+}
+
+
 void Fluxbox::eventLoop() {
     Display *disp = display();
     while (!m_shutdown) {
@@ -547,7 +597,9 @@ void Fluxbox::setupConfigFiles() {
         // create directory with perm 700
         if (mkdir(dirname.c_str(), 0700)) {
             fprintf(stderr, _FBTEXT(Fluxbox, ErrorCreatingDirectory,
-                                    "Can't create %s directory", "Can't create a directory, one %s for directory name"), dirname.c_str());
+                                    "Can't create %s directory", 
+                                    "Can't create a directory, one %s for directory name"), 
+                    dirname.c_str());
             cerr<<endl;
             return;
         }
@@ -605,13 +657,19 @@ void Fluxbox::handleEvent(XEvent * const e) {
                e->type == LeaveNotify) {
         m_last_time = e->xcrossing.time;
         m_mousescreen = searchScreen(e->xcrossing.root);
-    } else if (e->type == PropertyNotify)
+    } else if (e->type == PropertyNotify) {
         m_last_time = e->xproperty.time;
+        // check transparency atoms if it's a root pm
+        
+        BScreen *screen = searchScreen(e->xproperty.window);
+        if (screen) {
+            FbTk::FbPixmap::rootwinPropertyNotify(screen->screenNumber(), e->xproperty.atom);
+        }
+    }
 
     // we need to check focus out for menus before
     // we call FbTk eventhandler
     // so we can get FbTk::Menu::focused() before it sets to 0
-
     if (e->type == FocusOut &&
         e->xfocus.mode != NotifyGrab &&
         e->xfocus.detail != NotifyPointer &&
@@ -726,11 +784,13 @@ void Fluxbox::handleEvent(XEvent * const e) {
 #ifdef DEBUG
         cerr<<__FILE__<<"("<<__FUNCTION__<<"): MappingNotify"<<endl;
 #endif // DEBUG
-
-        FbTk::KeyUtil::instance().init(); // reinitialise the key utils
-        // reconfigure keys (if the mapping changes, they don't otherwise update
-        m_key->reconfigure(StringUtil::expandFilename(*m_rc_keyfile).c_str());
-
+        if (e->xmapping.request == MappingKeyboard
+            || e->xmapping.request == MappingModifier) {
+            XRefreshKeyboardMapping(&e->xmapping);
+            FbTk::KeyUtil::instance().init(); // reinitialise the key utils
+            // reconfigure keys (if the mapping changes, they don't otherwise update
+            m_key->reconfigure(StringUtil::expandFilename(*m_rc_keyfile).c_str());
+        }
         break;
     case CreateNotify:
 	break;
@@ -781,7 +841,6 @@ void Fluxbox::handleEvent(XEvent * const e) {
 
     } break;
     case LeaveNotify:
-
         m_last_time = e->xcrossing.time;
         break;
     case Expose:
@@ -1026,7 +1085,6 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
     if (keyScreen() == 0 || mouseScreen() == 0)
         return;
 
-
     switch (ke.type) {
     case KeyPress:
         m_key->doAction(ke);
@@ -1039,9 +1097,10 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
         if (m_watching_screen && m_watch_keyrelease) {
             // mask the mod of the released key out
             // won't mask anything if it isn't a mod
-            ke.state &= ~FbTk::KeyUtil::instance().keycodeToModmask(ke.keycode);
-
-            if ((m_watch_keyrelease & ke.state) == 0) {
+            unsigned int state = FbTk::KeyUtil::instance().isolateModifierMask(ke.state);
+            state &= ~FbTk::KeyUtil::instance().keycodeToModmask(ke.keycode);
+            
+            if ((m_watch_keyrelease & state) == 0) {
 
                 m_watching_screen->notifyReleasedKeys(ke);
                 XUngrabKeyboard(FbTk::App::instance()->display(), CurrentTime);
@@ -1057,8 +1116,6 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
     default:
         break;
     }
-
-
 }
 
 /// handle system signals
@@ -1072,13 +1129,13 @@ void Fluxbox::handleSignal(int signum) {
         waitpid(-1, 0, WNOHANG | WUNTRACED);
         break;
     case SIGHUP:
-        load_rc();
+        restart();
         break;
     case SIGUSR1:
-        reload_rc();
+        load_rc();
         break;
     case SIGUSR2:
-        rereadMenu();
+        reload_rc();
         break;
     case SIGSEGV:
         abort();
@@ -1281,11 +1338,13 @@ BScreen *Fluxbox::searchScreen(Window window) {
 
 AtomHandler* Fluxbox::getAtomHandler(const std::string &name) {
     if ( name != "" ) {
-        for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end(); it++ ) {
-            if ( name == (*it).second )
-                return (*it).first;
-        }
+        using namespace FbTk;
+        AtomHandlerContainerIt it = find_if(m_atomhandler.begin(),
+                                            m_atomhandler.end(),
+                                            Compose(bind2nd(equal_to<string>(), name),
+                                                    Select2nd<AtomHandlerContainer::value_type>()));
+        if (it != m_atomhandler.end())
+            return (*it).first;
     }
     return 0;
 }
@@ -1294,7 +1353,6 @@ void Fluxbox::addAtomHandler(AtomHandler *atomh, const std::string &name) {
 }
 
 void Fluxbox::removeAtomHandler(AtomHandler *atomh) {
-
     for (AtomHandlerContainerIt it= m_atomhandler.begin();
          it != m_atomhandler.end();
          ++it) {
@@ -1306,11 +1364,11 @@ void Fluxbox::removeAtomHandler(AtomHandler *atomh) {
 }
 
 WinClient *Fluxbox::searchWindow(Window window) {
-    std::map<Window, WinClient *>::iterator it = m_window_search.find(window);
+    WinClientMap::iterator it = m_window_search.find(window);
     if (it != m_window_search.end())
         return it->second;
 
-    std::map<Window, FluxboxWindow *>::iterator git = m_window_search_group.find(window);
+    WindowMap::iterator git = m_window_search_group.find(window);
     return git == m_window_search_group.end() ? 0 : &git->second->winClient();
 }
 
@@ -1521,22 +1579,18 @@ void Fluxbox::load_rc(BScreen &screen) {
         cerr<<__FILE__<<"("<<__FUNCTION__<<"): Workspaces="<<
             screen.getNumberOfWorkspaces()<<endl;
 #endif // DEBUG
-        char *search = StringUtil::strdup(value.addr);
-
-        int i;
-        for (i = 0; i < screen.getNumberOfWorkspaces(); i++) {
-            char *nn;
-
-            if (! i) nn = strtok(search, ",");
-            else nn = strtok(0, ",");
-
-            if (nn)
-                screen.addWorkspaceName(nn);
-            else break;
-
+        string values(value.addr);
+        BScreen::WorkspaceNames names;
+        
+        StringUtil::removeTrailingWhitespace(values);
+        StringUtil::removeFirstWhitespace(values);
+        StringUtil::stringtok<BScreen::WorkspaceNames>(names, values, ",");
+        BScreen::WorkspaceNames::iterator it;
+        for(it = names.begin(); it != names.end(); it++) {
+            if (!(*it).empty() && (*it) != "")
+            screen.addWorkspaceName((*it).c_str());
         }
-
-        delete [] search;
+        
     }
 
     FbTk::Image::removeAllSearchPaths();
@@ -1597,16 +1651,16 @@ void Fluxbox::reconfigure() {
 
 void Fluxbox::real_reconfigure() {
 
-    XrmDatabase new_blackboxrc = (XrmDatabase) 0;
+    XrmDatabase new_fluxboxrc = (XrmDatabase) 0;
 
     string dbfile(getRcFilename());
-    XrmDatabase old_blackboxrc = XrmGetFileDatabase(dbfile.c_str());
+    XrmDatabase old_fluxboxrc = XrmGetFileDatabase(dbfile.c_str());
 
-    XrmMergeDatabases(new_blackboxrc, &old_blackboxrc);
-    XrmPutFileDatabase(old_blackboxrc, dbfile.c_str());
+    XrmMergeDatabases(new_fluxboxrc, &old_fluxboxrc);
+    XrmPutFileDatabase(old_fluxboxrc, dbfile.c_str());
 
-    if (old_blackboxrc)
-        XrmDestroyDatabase(old_blackboxrc);
+    if (old_fluxboxrc)
+        XrmDestroyDatabase(old_fluxboxrc);
 
     ScreenList::iterator screen_it = m_screen_list.begin();
     ScreenList::iterator screen_it_end = m_screen_list.end();
@@ -1762,18 +1816,13 @@ void Fluxbox::setFocusedWindow(WinClient *client) {
 
     if (m_focused_window != 0) {
         // check if m_focused_window is valid
-        bool found = false;
-        std::map<Window, WinClient *>::iterator it = m_window_search.begin();
-        std::map<Window, WinClient *>::iterator it_end = m_window_search.end();
-        for (; it != it_end; ++it) {
-            if (it->second == m_focused_window) {
-                // we found it, end loop
-                found = true;
-                break;
-            }
-        }
-
-        if (!found) {
+        WinClientMap::iterator it = find_if(m_window_search.begin(),
+                                            m_window_search.end(),
+                                            Compose(bind2nd(equal_to<WinClient *>(), m_focused_window),
+                                                    Select2nd<WinClientMap::value_type>()));
+                
+        // if not found...
+        if (it == m_window_search.end()) {
             m_focused_window = 0;
         } else {
             old_client = m_focused_window;
@@ -1807,7 +1856,6 @@ void Fluxbox::setFocusedWindow(WinClient *client) {
         m_focused_window = 0;
 
 
-
     if (screen != 0) {
         screen->updateNetizenWindowFocus();
         for (AtomHandlerContainerIt it= m_atomhandler.begin();
@@ -1824,7 +1872,6 @@ void Fluxbox::setFocusedWindow(WinClient *client) {
              it != m_atomhandler.end(); it++)
             (*it).first->updateFocusedWindow(*old_screen, 0);
     }
-
 }
 
 /**
@@ -1911,12 +1958,15 @@ void Fluxbox::unfocusWindow(WinClient &client, bool full_revert, bool unfocus_fr
 
 
 void Fluxbox::watchKeyRelease(BScreen &screen, unsigned int mods) {
+    
     if (mods == 0) {
         cerr<<"WARNING: attempt to grab without modifiers!"<<endl;
         return;
     }
     m_watching_screen = &screen;
-    m_watch_keyrelease = mods;
+
+    // just make sure we are saving the mods with any other flags (xkb)
+    m_watch_keyrelease = FbTk::KeyUtil::instance().isolateModifierMask(mods);
     XGrabKeyboard(FbTk::App::instance()->display(),
                   screen.rootWindow().window(), True,
                   GrabModeAsync, GrabModeAsync, CurrentTime);
