@@ -1,4 +1,4 @@
-// Copyright (c) 2002 Henrik Kinnunen (fluxgen at linuxmail.org)
+// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
 // Copyright (c) 1997 - 2000 Brad Hughes <bhughes at trolltech.com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -18,12 +18,13 @@
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 
-// $Id: bsetroot.cc,v 1.20 2003/08/12 01:34:13 fluxgen Exp $
+// $Id: bsetroot.cc 3867 2005-01-24 18:49:01Z mathias $
 
 #include "bsetroot.hh"
 
-#include "../src/I18n.hh"
+#include "../src/FbTk/I18n.hh"
 #include "../src/FbTk/ImageControl.hh"
+#include "../src/FbTk/GContext.hh"
 #include "../src/FbRootWindow.hh"
 
 #ifdef HAVE_CONFIG_H
@@ -32,9 +33,21 @@
 
 #include <X11/Xatom.h>
 
-#include <cstring>
-#include <cstdlib>
-#include <cstdio>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 #include <iostream>
 
 using namespace std;
@@ -106,12 +119,11 @@ bsetroot::bsetroot(int argc, char **argv, char *dpy_name)
     }
 
     if ((mod + sol + grd) != true) {
-        fprintf(stderr,
-                I18n::instance()->
-                getMessage(
-                           FBNLS::bsetrootSet, FBNLS::bsetrootMustSpecify,
-                           "%s: error: must specify on of: -solid, -mod, -gradient\n"),
-                m_app_name);
+        _FB_USES_NLS;
+        cerr<<m_app_name<<
+            _FBTEXT(bsetroot, MustSpecify, 
+                    "Error: must specify one of: -solid, -mod, -gradient\n",
+                    "user didn't give one of the required options")<<endl;
 
         usage(2);
     }
@@ -140,8 +152,7 @@ bsetroot::~bsetroot() {
     }
 
     if (img_ctrl != 0) {
-        int i = 0;
-        for (; i < num_screens; i++)
+        for (int i=0; i < num_screens; i++)
             delete img_ctrl[i];
 
         delete [] img_ctrl;
@@ -165,15 +176,12 @@ void bsetroot::setRootAtoms(Pixmap pixmap, int screen) {
 
     // doing this to clean up after old background
     if (atom_root != None && atom_eroot != None) {
-
-        XGetWindowProperty(display(), root.window(),
-                           atom_root, 0L, 1L, false, AnyPropertyType,
-                           &type, &format, &length, &after, &data_root);
+        root.property(atom_root, 0L, 1L, false, AnyPropertyType,
+                      &type, &format, &length, &after, &data_root);
 
         if (type == XA_PIXMAP) {
-            XGetWindowProperty(display(), root.window(),
-                               atom_eroot, 0L, 1L, False, AnyPropertyType,
-                               &type, &format, &length, &after, &data_eroot);
+            root.property(atom_eroot, 0L, 1L, False, AnyPropertyType,
+                          &type, &format, &length, &after, &data_eroot);
                 
             if (data_root && data_eroot && type == XA_PIXMAP &&
                 *((Pixmap *) data_root) == *((Pixmap *) data_eroot)) {
@@ -187,15 +195,14 @@ void bsetroot::setRootAtoms(Pixmap pixmap, int screen) {
     atom_eroot = XInternAtom(display(), "ESETROOT_PMAP_ID", false);
 
     if (atom_root == None || atom_eroot == None) {
-        cerr<<"couldn't create pixmap atoms, giving up!"<<endl;
+        _FB_USES_NLS;
+        cerr<<_FBTEXT(bsetroot, NoPixmapAtoms, "Couldn't create pixmap atoms, giving up!", "Couldn't create atoms to point at root pixmap")<<endl;
         exit(1);
     }
 
     // setting new background atoms
-    XChangeProperty(display(), root.window(),
-                    atom_root, XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &pixmap, 1);
-    XChangeProperty(display(), root.window(),
-                    atom_eroot, XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &pixmap, 1);
+    root.changeProperty(atom_root, XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &pixmap, 1);
+    root.changeProperty(atom_eroot, XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &pixmap, 1);
 
 }
 
@@ -208,38 +215,28 @@ void bsetroot::solid() {
     pixmaps = new Pixmap[ScreenCount(display())];
 
     for (; screen < ScreenCount(display()); screen++) {
-        FbTk::Color c;
-        if (!c.setFromString(fore, screen)) // just report error
-            continue;
-
-        FbRootWindow root(screen);
-
-        GC gc;
-        XGCValues gcv;
 
+        FbTk::Color c(fore, screen);
         if (! c.isAllocated())
             c.setPixel(BlackPixel(display(), screen));
 
-        gcv.foreground = c.pixel();
-        gc = XCreateGC(display(), root.window(),
-                       GCForeground , &gcv);
+        FbRootWindow root(screen);
+
+        FbTk::GContext gc(root);
+        gc.setForeground(c);
 
         pixmaps[screen] = XCreatePixmap(display(), 
                                         root.window(),
                                         root.width(), root.height(),
                                         root.depth());
 
-        XFillRectangle(display(), pixmaps[screen], gc, 0, 0,
+        XFillRectangle(display(), pixmaps[screen], gc.gc(), 0, 0,
                        root.width(), root.height());
 
         setRootAtoms(pixmaps[screen], screen);
 
-        XSetWindowBackgroundPixmap(display(),
-                                   root.window(), pixmaps[screen]);
-
-        XClearWindow(display(), root.window());
-
-        XFreeGC(display(), gc);
+        root.setBackgroundPixmap(pixmaps[screen]);
+        root.clear();
     }
 }
 
@@ -274,10 +271,9 @@ void bsetroot::modula(int x, int y) {
             }
         }
 
-        FbTk::Color f, b;
-        GC gc;
+
         Pixmap bitmap, r_bitmap;
-        XGCValues gcv;
+
 
         bitmap = XCreateBitmapFromData(display(),
                                        root.window(), data, 16, 16);
@@ -287,43 +283,37 @@ void bsetroot::modula(int x, int y) {
                                  root.window(), 16, 16,
                                  root.depth());
 
-        f.setFromString(fore, screen);
-        b.setFromString(back, screen);
+        FbTk::Color f(fore, screen), b(back, screen);
 
         if (! f.isAllocated())
             f.setPixel(WhitePixel(display(), screen));
         if (! b.isAllocated())
             b.setPixel(BlackPixel(display(), screen));
 
-        gcv.foreground = f.pixel();
-        gcv.background = b.pixel();
+        FbTk::GContext gc(root);
 
-        gc = XCreateGC(display(), root.window(),
-                       GCForeground | GCBackground, &gcv);
+        gc.setForeground(f);
+        gc.setBackground(b);
 
         // copying bitmap to the one going to be used as tile
-        XCopyPlane(display(), bitmap, r_bitmap, gc,
+        XCopyPlane(display(), bitmap, r_bitmap, gc.gc(),
                    0, 0, 16, 16, 0, 0, 1l);
 
-        XSetTile(display(), gc, r_bitmap);
-        XSetFillStyle(display(), gc, FillTiled);	
+        gc.setTile(r_bitmap);
+        gc.setFillStyle(FillTiled);
 
         pixmaps[screen] = XCreatePixmap(display(), 
                                         root.window(),
                                         root.width(), root.height(),
                                         root.depth());
 
-        XFillRectangle(display(), pixmaps[screen], gc, 0, 0,
+        XFillRectangle(display(), pixmaps[screen], gc.gc(), 0, 0,
                        root.width(), root.height());
 
         setRootAtoms(pixmaps[screen], screen);
+        root.setBackgroundPixmap(pixmaps[screen]);
+        root.clear();
 
-        XSetWindowBackgroundPixmap(display(),
-                                   root.window(), pixmaps[screen]);
-
-        XClearWindow(display(), root.window());
-
-        XFreeGC(display(), gc);
         XFreePixmap(display(), bitmap);
         XFreePixmap(display(), r_bitmap);
     }
@@ -338,51 +328,50 @@ void bsetroot::gradient() {
     // as the pixmap has been destroyed
     Pixmap tmp;
     pixmaps = new Pixmap[ScreenCount(display())];
-
+    // we must insert gradient text
+    string texture_value = grad ? grad : "solid";
+    texture_value.insert(0, "gradient ");
+    FbTk::Texture texture;
+    texture.setFromString(texture_value.c_str());
+    
     for (int screen = 0; screen < ScreenCount(display()); screen++) {
         FbRootWindow root(screen);
 
-        FbTk::Texture texture;
-        GC gc;
-        XGCValues gcv;
 
-        texture.setFromString(grad);
+        FbTk::GContext gc(root);
         texture.color().setFromString(fore, screen);
         texture.colorTo().setFromString(back, screen);
-		
+
+
         if (! texture.color().isAllocated())
             texture.color().setPixel(WhitePixel(display(), screen));
+
         if (! texture.colorTo().isAllocated())
             texture.colorTo().setPixel(BlackPixel(display(), screen));
 
-        tmp = img_ctrl[screen]->renderImage(root.width(),
-                                            root.height(), texture);
+        tmp = img_ctrl[screen]->renderImage(root.width(), root.height(), 
+                                            texture);
 
         pixmaps[screen] = XCreatePixmap(display(), 
                                         root.window(),
                                         root.width(), root.height(),
                                         root.depth());
 
-        gc = XCreateGC(display(), root.window(),
-                       GCForeground , &gcv);
-
-        XCopyArea(display(), tmp, pixmaps[screen], gc, 0, 0,
+        
+        XCopyArea(display(), tmp, pixmaps[screen], gc.gc(), 0, 0,
                   root.width(), root.height(),
                   0, 0);
 
         setRootAtoms(pixmaps[screen], screen);
 
-        XSetWindowBackgroundPixmap(display(),
-                                   root.window(), pixmaps[screen]);
-
-        XClearWindow(display(), root.window());
+        root.setBackgroundPixmap(pixmaps[screen]);
+        root.clear();
 
         if (! (root.visual()->c_class & 1)) {
             img_ctrl[screen]->removeImage(tmp);
             img_ctrl[screen]->cleanCache();
         }
 
-        XFreeGC(display(), gc);
     }
 }
 
@@ -390,23 +379,21 @@ void bsetroot::gradient() {
  Shows information about usage
 */
 void bsetroot::usage(int exit_code) {
-    fprintf(stderr,
-            I18n::instance()->getMessage(
-                                         FBNLS::bsetrootSet, FBNLS::bsetrootUsage,
-                                         "%s 2.2 : (c) 2003 Fluxbox Development Team\n"
-                                         "%s 2.1 : (c) 2002 Claes Nasten\n"
-                                         "%s 2.0 : (c) 1997-2000 Brad Hughes\n\n"
-                                         "  -display <string>        display connection\n"
-                                         "  -mod <x> <y>             modula pattern\n"
-                                         "  -foreground, -fg <color> modula foreground color\n"
-                                         "  -background, -bg <color> modula background color\n\n"
-                                         "  -gradient <texture>      gradient texture\n"
-                                         "  -from <color>            gradient start color\n"
-                                         "  -to <color>              gradient end color\n\n"
-                                         "  -solid <color>           solid color\n\n"
-                                         "  -help                    print this help text and exit\n"),
-            m_app_name, m_app_name);
- 
+    _FB_USES_NLS;
+    cerr<<m_app_name<<" 2.3 : (c) 2003-2004 Fluxbox Development Team"<<endl;
+    cerr<<m_app_name<<" 2.1 : (c) 2002 Claes Nasten"<<endl;
+    cerr<<m_app_name<<" 2.0 : (c) 1997-2000 Brad Hughes\n"<<endl;
+    cerr<<_FBTEXT(bsetroot, Usage,
+                  "  -display <string>        display connection\n"
+                  "  -mod <x> <y>             modula pattern\n"
+                  "  -foreground, -fg <color> modula foreground color\n"
+                  "  -background, -bg <color> modula background color\n\n"
+                  "  -gradient <texture>      gradient texture\n"
+                  "  -from <color>            gradient start color\n"
+                  "  -to <color>              gradient end color\n\n"
+                  "  -solid <color>           solid color\n\n"
+                  "  -help                    print this help text and exit\n",
+                  "bsetroot usage options")<<endl;
     exit(exit_code);
 }
 
@@ -415,18 +402,18 @@ int main(int argc, char **argv) {
     char *display_name = (char *) 0;
     int i = 1;
   
-    NLSInit("fluxbox.cat");
+    FbTk::NLSInit("fluxbox.cat");
   
     for (; i < argc; i++) {
         if (! strcmp(argv[i], "-display")) {
             // check for -display option
 
             if ((++i) >= argc) {
-                fprintf(stderr,
-                        I18n::instance()->getMessage(
-                                                     FBNLS::mainSet, FBNLS::mainDISPLAYRequiresArg,
-                                                     "error: '-display' requires an argument\n"));
-	
+                _FB_USES_NLS;
+                cerr<<_FBTEXT(main, DISPLAYRequiresArg,
+                              "error: '-display' requires an argument",
+                              "option requires an argument")<<endl;
+
                 ::exit(1);
             }
 
